/*
   Tony Weir Jan 89
   Functions to determine if user is able to access a non-chargeable
   printer.
   
   Current strategy is 

1.     If call originates from castle check erccreg - if fourth letter is
     U or V then check the per user printer authentication file,
     otherwise let it through.  This allows specified ungrads or postgrads
     to use the printers.

2.     If call comes from another machine then check if it is in the
     host printer authentication file which contains a list of hosts
     from which all users are permitted to use non-cahrgeable printers.

   The host printer authentication file simply contains a list of hosts
   separated by newlines.

*/


#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <string.h>
#include <malloc.h>
#include <fcntl.h>

#define TRUE  1
#define FALSE 0

#define NAME_LENGTH 9
#define MACHINE_LENGTH 81

/* structure to hold the user printer file internally */
static struct user_entry {
	char	name[9];
	char	machine[30];
};


static struct hosttype {
	char	host[MACHINE_LENGTH];
};


static int	numentries;          /* nasty this eh ? */
static int	numuentries;         /* number of entries in user printer file */
static int	numhentries;         /* number of entries in host printer file */
static struct user_entry *user; /* array to hold user printer file */
static struct hosttype *host; /* array to hold host printer file */
static char	*pp, *tp, *endp;     /* pointers to decompose a user printer entry into its component fields */
static char	*addr;               /* pointer to initial read of file */

/* read the user printer file into user and sort ascendingly by name/machine */

int	read_file(filename)
char	*filename;
{

	static int	fp;		/* file descriptor  printer file */
	int comment;
	struct stat statbuff;	/* status of  file */

	if ((fp = open(filename, O_RDONLY)) < 0) {
		perror(filename);
		return(0);
	}

	if (fstat(fp, &statbuff) != 0) {
		perror(filename);
		return(0);
	}

	if ((addr = malloc((unsigned)statbuff.st_size + 1)) == NULL) {
			(void)fprintf(stderr, "malloc failed\n");
			return(0);
	}

	if (read(fp, addr, (int)statbuff.st_size) != statbuff.st_size) {
		perror(filename);
		return(0);
	}

	/* the user printer authentication file is now in memory */

	endp = addr + statbuff.st_size;
	if (close(fp) == -1) {
		perror(filename);
		return(0);
	}

	tp = addr;

	/* count number of entries */
	numentries = 0;
	comment = 0;
	while (tp < endp) {
		if (*tp == '#') 
			comment = 1;
		if (*tp == '\n') {
			if (comment == 0)
				numentries++;
			else
				comment = 0;
		}
		tp++;
	}
	return(TRUE);
}


#define HOST_FILE "/usr/local/etc/charged_printers/hosts"
#define USER_FILE "/usr/local/etc/charged_printers/users"

int read_host_file()
{
	struct hosttype *current; /* host currently processed */
	int	pos;              /* loop counter */
	void skip();              /* function used to decompose host printer file entry */


	if (read_file(HOST_FILE) == FALSE)
		return(FALSE);

	numhentries = numentries;  /* one to remember */
	host  = (struct hosttype *)malloc((unsigned)sizeof(struct hosttype ) * numhentries);
	/* construct user array from internally held file */
	pp = addr;


	for (pos = 0; pos < numhentries; pos++) {
		current = &host[pos];
		skip(); 
		(void)strncpy(current->host, tp, MACHINE_LENGTH);
	}
	free(addr);

	return(TRUE);
}


int
read_user_file()
{
	struct user_entry *current;	/* the user entry currently processed */
	int	pos;			/* loop counter */
	void skip();			/* function used to decompose user printer file entry */
	int	sort_cmp();		/* function used for sort comparison */


	if (read_file(USER_FILE) == FALSE)
		return(FALSE);

	numuentries = numentries;
	user = (struct user_entry *)malloc((unsigned)sizeof(struct user_entry ) * numuentries);
	/* construct user array from internally held file */
	pp = addr;
	for (pos = 0; pos < numuentries; pos++) {
		current = &user[pos];
		skip(); 
		(void)strncpy(current->name, tp, NAME_LENGTH);
		skip(); 
		(void)strncpy(current->machine, tp, MACHINE_LENGTH);
	}
	free(addr);
	qsort((char *)user, numuentries, sizeof(struct user_entry ), sort_cmp);

	return(TRUE);
}



/* skip positions tp and pp in positions, and with values, which
 * allow the memory held printer file to be decomposed into its component
 * fields, fields may be separated by a colon or a newline.
 */
void skip()
{
	tp = pp;
	while (*pp != ':' && *pp != '\n' && pp < endp) 
		pp++;
	*pp++ = NULL;
	while (*pp == '\n' && *pp == ':') 
		pp++;/* skip over new lines */
}




/* sort comparison function to sort by name and machine */
int	sort_cmp(u1, u2)
struct user_entry *u1, *u2;
{
	int	result;

	if ( (result = strcmp(u1->name, u2->name)) == 0)
		return(strcmp(u1->machine, u2->machine));
	else
		return(result);
}


int	validate_user(uname, hostname)
char	*uname, *hostname;
{
	char *cp;

	cp = hostname;
	while (*cp && (*cp != ' ')) {
		cp++;
	}
	*cp = '\0';
	return(gethost(hostname));
}


/* Determine if all users from this host are allowed to access non-chargeable
 * queues.
 */

int	gethost(hostname)
char	*hostname;
{
	int	pos;
	struct hosttype *current;

	if (read_host_file() == FALSE) {
		(void)fprintf(stderr, "Failed to read %s \n", HOST_FILE);
		return(FALSE);
	}

	for (pos = 0; pos < numhentries; pos++) {
		current = &host[pos];
		if (strncmp(current->host, hostname, MACHINE_LENGTH) == 0)
			return(TRUE);
	}

	return(FALSE);
}
