/*
 ===========================================================================
 =                                                                         =
 =  (C) Copyright 1995, Computer Technology Institute (CTI)                =
 =                                                                         =
 =  Permission to use, copy, modify and distribute this software and       =
 =  its documentation for non-commercial use and without fee is hereby     =
 =  granted, provided that the above copyright notice appear in all        =
 =  copies and that both that copyright notice and this permission         =
 =  notice and warranty disclaimer appear in supporting documentation,     =
 =  and that the name of Computer Technology Institute (CTI) or any of     =
 =  its entities not be used in advertising or publicity pertaining to     =
 =  distribution of the software and its documentation without specific    =
 =  prior permission.                                                      =
 =                                                                         =
 =  Computer Technology Institute (CTI) disclaims all warranties with      =
 =  regard to this software and makes no representations about the         =
 =  suitability of this software and its documentation for any purpose.    =
 =  It is provided "as is" without expressed or implied warranty.          =
 =                                                                         =
 ===========================================================================
 =                                                                         =
 =                        Project CTI-Print                                =
 =                                                                         =
 = File:                                                                   =
 =   ifhpiii.c                                                             =
 =                                                                         =
 = Synopsis:                                                               =
 =   This is the main driver module which handles the printer interface.   =
 =                                                                         =
 = Author:                                                                 =
 =   Panos Dimakopoulos, Systems Programmer,                                =
 =   Computer Technology Institute,                                        =
 =   Division of Computing Facilities,                                     =
 =   P.O. Box 1122,                                                        =
 =   261 10  Patras,                                                       =
 =   Greece                                                                =
 =   (e-mail: dimakop@cti.gr)                                              =
 =   Tel: +30 61 992061                                                    =
 =   Fax: +30 61 993973                                                    =
 =                                                                         =
 ===========================================================================
*/


/****************************************************************************
 * Modification History:
 * 	Revision 1.1	95/01/12	11:45:36
 *		 Creation.
 * 	Revision 1.2	95/01/30	17:16:24
 *		 readline() owner's mention.
 * 	Revision 1.3	95/01/30	17:51:57
 *		 Copyright label and file name's size changed.
 * 	Revision 1.4	95/02/02	13:39:49
 *		 -B"<banner page title>" flag added.
 * 	Revision 1.5	95/02/03	18:51:41
 *		 Nulls and sleep time decreased for speed.
 * 	Revision 1.6	95/02/09	20:17:46
 *		 -B"<banner page title>" flag can have a \n embedded to be
 *		 printed underneath the Institution's title.
 * 	Revision 1.7	95/02/10	17:39:55
 *		 Full CTI address.
 * 	Revision 1.8	95/02/14	14:04:11
 *		 New Copyright notice.
 * 	Revision 1.9	95/02/14	19:16:33
 *		 Copyright 1994 becomes 1995.
 * 	Revision 1.10	95/05/30	17:52:05
 *		 Version 1.1 initiated.
 * 	Revision 1.11	95/08/22	15:01:07
 *		 Version 1.2 initiated and Porting to Solaris.
 *
 */


#ifdef SVR4
#include <sys/ddi.h>
#endif
#include <stdio.h>
#include <errno.h>
#include <signal.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/fcntl.h>
#include <ctype.h>
#include <unistd.h>
#include <string.h>
#include <hp4.h>

static char idallf[] = "@(#) Computer Technology Institute, CTI-Print Project";
static char idhp4[] = "@(#)       Filter for HP LaserJet 4, V 1.2 \n" ;
static char idbphp4[] = "@(#) bphp4.c 95/08/22 - V 1.11" ;

extern int errno;

char    sendline[MAXLINE+1];
char    readprin[MAXLINE+1];
char    job_start[100];
char    job_end[100];

		
#ifdef SVR4
#define CLEARBUF	memset(readprin, 0, MAXLINE+1)
#else
#define CLEARBUF	bzero(readprin, MAXLINE+1)
#endif
#define DIE		1

char *name;		/* name of filter */

/* 
 * The following variables are set by the arguments passed to the filter
 * from the caller (lpd)
 */
	char	*zopts, 
		*class, 
		*job, 
		*login, 
		*accntname, 
		*host, 
		*accntfile, 
		*bantitle = "Computer Technology Institute\n\
Division of Computing Facilities", 
		*format,
		*printer, 
		*filename, 
                *date,
                *username,
                *seq_number,
		*pr_commnt;
	int	width, 
		length, 
		xwidth, 
		ylength,
		debug, 
		literal, 
		indent;

int	prmsline,			/* line of printer_perms to change  */
	errorcode	= FILTABORT,	/* status the errlog funcs exit with */
	mode		= 0,		/* PCL or Postscript */
	npages		= 1, 		/* number of pages */
	pipefd[2],			/* Pipe between father and child which
					 * monitors the printer port */
	alrmflag,			/* Set by ALARM signal handling funcs */
	monitpid	=-1, 		/* pid of the port monitoring child */
	printer_ok,
	interrupted	= FALSE,	/* Set when signal received	*/
	job_started 	= FALSE,	/* Flag for the successful or not 
					 * initiation of the job */
	job_ended 	= FALSE;	/* Flag for the successful or not 
					 * termination of the job */
char	*query[3];			/* Holds the every time pjl commands 
					 * sent to the printer port */

static int	wait_time	= 30,	/* Time to wait between two retries 
					   (secs) */
		retries		= 0;	/* Number of retries if printer comes 
					   off-line 0 means for ever */

/******************************************************************************
 *	PCL Cover Description strings
 */
static char pclhead10[] = { 
	"\033*p30x50Y\033*c1800a100b45g2P"
} ;

static char pclhead12[] = {
	"\033(0U\033(s1p25v1s0b5T\033*p100x350Y%s"
} ;

static char pclhead13[] = {
	"\033(0U\033(s1p18v1s0b5T\033*p100x450Y%s"
} ;

static char pclhead2[]  = { 
	"\033*p30x560Y\033*c1800a100b25g2P\033(8U\033(s1p24v0s0b4148T" 
} ;

static char pclhead3[] 	= {
	"\033*p500x2900Y%40s\033(0N\033(s1p12v0s0b4T\033*p50x3200Y\251 \
\033(s9V\033*p80x3186Y \
1995, Computer Technology Institute (CTI)\033*p100x800Y\033(s18v3BJob:\033(s0B"
} ;

static char pclhead4[] = {
	"\033&dD %s \033&d@"
} ;

static char pclhead5[] = {
	" %s \033(s1Sfrom node:\033(s0S %s"
} ;

static char pclhead6[] = { 
	"\033*p300x1200Y\033(s56V%s\033*p100x1600Y\033(s24v3BFile:\033(s24v0B \
%s\033*p100x2000Y\033(s18v3BDate:\033(s0B %s"
} ;


main(argc,argv)
int argc;
char *argv[];
{
	int	printer_code, k,aa;
	char 	s[150];
	void 	cleanup(), readstatus(), pipeprob();
	void	signreceiv(), nochildren();
	int 	readpipe();
	short 	wrap();

	debug = 2;
	get_retries();
	getargs( argc, argv );
	getpjlargs();

	(void *)signal(SIGCHLD, nochildren);
	(void *)signal(SIGINT, cleanup);
	(void *)signal(SIGHUP, cleanup);
	(void *)signal(SIGTERM, cleanup);
	(void *)signal(SIGQUIT, cleanup);
	(void *)signal(SIGBUS, cleanup);
	(void *)signal(SIGSEGV, cleanup);
	(void *)signal(SIGPIPE, cleanup);

	forkport();

   	/*
	* Try to synchronise with printer
	*/
	if ( pr_synch(STDOUT)==FALSE ) {
			log(2,"Failed to synchronise with printer");
			fexit(FILTRETRY);
	} else	log(2,"Synchronised with printer");

   	/*
	* Give status info only when you are asked to
	*/
	pr_ustatusoff(STDOUT);

       /*
	* Check printer status
	*/
	printer_ok = FALSE;
	do {
		switch ( printer_code=pr_status(STDOUT) )	{
			case 10001:		/* Printer ready */
				log(2,"-- Printer ready");
				printer_ok = TRUE;
				break;
			case 10003:		/* Printer warming up */
				log(2,"-- Printer warming up");
				sleep(45);	/* Warming up max time */
				break;
			case 10000:		/* Printer in powersave mode */
			case 35078:		
				log(2,"-- Printer in powersave");
				wake_up();
				/* sleep(45);	Warming up max time */
				break;
			case 10006:		/* Printer ready */
				log(2,"-- Toner low but carry on!");
				printer_ok = TRUE;
				break;
			default:
				check_code( printer_code, stderr );
				sleep(1);
				break;
		}
	} while	( printer_ok==FALSE );

	pr_ustatus(STDOUT,"@PJL USTATUS JOB = ON\r\n");
	
	pr_startjob(STDOUT);

	pr_ustatus(STDOUT,"@PJL USTATUS DEVICE = VERBOSE\r\n");
	pr_timed(STDOUT);
	setpcl_on();
	banner();
	pclresetprinter();

	pr_endjob(STDOUT);
	check_job_end();

	updaccnt(npages);
	doaccnt();

	close(STDOUT);

	fexit(FILTSUCC);
}

banner( )
{
	char *Time_str(), *s2;

	sprintf(sendline,"%s",pclhead10);
	Out_line();
	if ( (s2=strchr(bantitle, '\n')) != NULL ) {
		*s2 = '\0' ;
		s2++ ;
		sprintf(sendline,pclhead13,s2);
		Out_line();
	}
	sprintf(sendline,pclhead12,bantitle);
	Out_line();
	sprintf(sendline,"%s",pclhead2);
	Out_line();
	sprintf(sendline,pclhead3,pr_commnt);
	Out_line();
	sprintf(sendline,pclhead5,job,host) ;
	Out_line();
	sprintf(sendline,pclhead6,login,filename,Time_str()) ;
	Out_line();
}


/*
 * Write the buffer out to the printer.
 */
Out_line()
{
	int i;
	i = strlen(sendline);

	if (writecn( fileno(stdout), sendline, i ) != i)
			logerr(3,"Out_line: <%s> writecn failed\n", sendline );
}

int writecn(fd,ptr,nbytes)
register int fd;
register char *ptr;
int nbytes;
{
	int nleft, nwritten;
	void (*f)(), wait_printer();
	
	nleft = nbytes;
	while (nleft > 0) {
		f=signal(SIGALRM,wait_printer);
		alarm(3);
		nwritten = write (fd, ptr, nleft);
		alarm(0);
		(void *)signal(SIGALRM,f);
		if (nwritten == 0) {
			logerr(4,"writecn wrote nothing");
			continue ;
		}
		if (nwritten < 0) {
			logerr(4,"writecn failure");
			if (errno==EINTR) {
				log(4,"Due to signal");
				return(-1);
			} else
				fexit(FILTRETRY);
		}
		nleft -= nwritten;
		ptr += nwritten;
	}
	return(nbytes - nleft);
}

void cleanup(sig)
int sig;
{
	log(4,"Signal %u received-entering cleanup", sig);

	switch (sig) {
		case SIGINT:		/* Sent by lpd to terminate job */
			log(3,"Job Interrupted... Job removed");
			if ( mode==POSTSCRIPT ) {
				/*
				 * Send PostScript Interrupt/EOF chars
				 */
				writecn(STDOUT,"\003", 1);  /* ctrl-C */
				writecn(STDOUT,"\004", 1);  /* ctrl-D */
			}
			else if ( mode==PCL )
				pclresetprinter();  /* Send EscE char */
			/*
			 * If mode is not set to any of the two values
			 * PCL/POSTSCRIPT we do not care for resetting
			 * as no printing has taken place.
			 */
			fexit(FILTABORT);
		case SIGCHLD:
		case SIGTERM:
		case SIGHUP:
		case SIGQUIT:
			log(3,"Job Interrupted but job remains...");
			if ( mode==POSTSCRIPT ) {
				/*
				 * Send PostScript Interrupt/EOF chars
				 */
				writecn(STDOUT,"\003", 1);  /* ctrl-C */
				writecn(STDOUT,"\004", 1);  /* ctrl-D */
			}
			else if ( mode==PCL )
				pclresetprinter();  /* Send EscE char */
			/*
			 * If mode is not set to any of the two values
			 * PCL/POSTSCRIPT we do not care for resetting
			 * as no printing has taken place.
			 */
			fexit(FILTRETRY);
		case SIGSEGV:
		case SIGBUS:
			signal(SIGCHLD,SIG_DFL);
				/* Otherwise father receives SIGCHLD as well
				 * when he kills the child.
				 */
		case SIGPIPE:
			log(3,"Job Interrupted but job remains...");
			fexit(FILTRETRY);
		default:
			log(4,"Signal Ignored");
			return;
	}
}

static char null_str[]={"\000\000\000\000\000\000\000\000"};
#define NULL_LEN	8

wake_up()
{
	short cou;
	static int   woken=FALSE;

#define WAKEUP_STR	"\033(8U\033(s1p24v0s0b4148T\033*p100x900YWarming Up \
\033*p100x1100Ythe Printer"

	log(2,"Warming Up Process");
	if ( woken==TRUE ) {
		log(4,"But printer already awake...");
		return;
	}
	woken = TRUE ;

	write_check(STDOUT, PCLON, "Cannot set PCL Mode", DIE);
	for ( cou=0; cou<1000; cou++ )
		if ( writecn(STDOUT,null_str,NULL_LEN) != NULL_LEN )
			logerr_die(3,"Cannot send nulls");
	sleep(10);
	write_check(STDOUT, WAKEUP_STR, "Cannot send WAKEUP string", DIE);
	sleep(5);
	pclresetprinter();
	write_check(STDOUT, UELPJL, "Cannot send UELPJL", DIE);
}

/*
 * The transition is a bit odd. It needs to be accompanied with some
 * nulls and the driver hold off for some time. After experiments 
 * on HP 4M+ the optimum values have been found and used in next
 * functions.
 * See comments before setpcl_on() in file ../ifhp4/ifhp4.c
 */
setpcl_on()
{
	int cou;

	write_check(STDOUT, EOFSTR, "Cannot send EOF", DIE);
	resetprinter();
	for (cou=0;cou<30;cou++)
		if ( writecn(STDOUT,null_str,NULL_LEN) != NULL_LEN )
			logerr_die(3,"Cannot send nulls");
	write_check(STDOUT, PCLON, "Cannot set PCL Mode", DIE);
	sleep(3);
}

eofprinter()
{
	write_check(STDOUT, EOFSTR, "Cannot send EOF char", 0);
}

resetprinter()
{
	write_check(STDOUT, UEL, "Cannot UEL reset printer", 0);
}

pclresetprinter()
{
	write_check(STDOUT, PCLRESETSTR, "Cannot PCL reset printer", 0);
}

/*
 * The following function has been copied from a printer driver 
 * written by Lawrence LeMay (lemay@epx.cis.umn.edu).
 */
/*
 * Read a line from a descriptor.  Read the line one byte at a time,
 * looking for '\015' ('\r') (FF is the terminator for all responses returned 
 * from the series 4 printers).  We store the FF in the buffer,
 * then follow it with a null (the same as fgets(3)).
 * We return the number of characters up to, but not including,
 * the null (the same as strlen(3)).
 */

int
port_readline(fd, ptr, maxlen)
register int    fd;
register char   *ptr;
register int    maxlen;
{
    int                      n, rc;
    char                     c;
    static struct timeval    timeout;
    fd_set                   readfds;

    timeout.tv_sec = 1;
    timeout.tv_usec = 0;

    for (n = 0; n < maxlen; n++) {
        FD_ZERO(&readfds);
        FD_SET(fd, &readfds);
        if (select(fd+1, &readfds, (fd_set *) 0, (fd_set *) 0,
                &timeout) < 0) {
		if (errno==EINTR)	{
			log(4,"Select received signal");
			n-- ;
			continue;
		}
        	logerr(1,"select error");
        	fexit(FILTRETRY);
        }

        if (!FD_ISSET(fd, &readfds)) break;

        if ( (rc = read(fd, &c, 1)) == 1) {
		*ptr++ = c;
		if (c == '\014') {	/* ^L returned from printer */
			n++ ;
			break;
		}
        } else if (rc == 0) {
            if (n == 0)
                return(0);    /* EOF, no data read */
            else
                break;        /* EOF, some data was read */
        } else {
            return(-1);    /* error */
	}
    }

    *ptr = 0;
    return(n);
}

/*
 * Read a line from a descriptor.  Read the line one byte at a time,
 * looking for a newline.  We store the newline in the buffer,
 * then follow it with a null (the same as fgets(3)).
 * We return the number of characters up to, but not including,
 * the null (the same as strlen(3)).
 */

int
file_readline(fd, ptr, maxlen)
register int    fd;
register char   *ptr;
register int    maxlen;
{
    int                      n, rc;
    char                     c;

    for (n = 0; n < maxlen; n++) {

        if ( (rc = read(fd, &c, 1)) == 1) {
		*ptr++ = c;
		if (c == '\n') {
			n++ ;
			break;
		}
        } else if (rc == 0) {
            if (n == 0)
                return(0);    /* EOF, no data read */
            else
                break;        /* EOF, some data was read */
        } else {
            return(-1);    /* error */
	}
    }

    *ptr = 0;
    return(n);
}

/*
 *  doaccnt()
 *  writes the accounting information to the accounting file
 *  This has the format: user host printer pages format date
 */
doaccnt()
{
	time_t t, time();
	char *ctime();
	FILE *f;

	t = time((time_t *)0);
	if(accntfile && access(accntfile, W_OK) >= 0 &&
	    (f = fopen(accntfile, "a" )) != NULL) {
		fprintf(f,"%s\t%s\t%s\t%s\t%7d\t%s\t%s",
			login? login: "NULL", 
			host? host: "NULL", 
			printer? printer: "NULL", 
			filename? filename: "NULL",
			npages,
			format? format: "NULL", 
			ctime(&t));
		(void)fclose(f);
	}
}


readpipe(inptr)
int *inptr;
{
	char *buf, *getcrstr();
	int   n, cd, get_num();
	void wait_printer();

	CLEARBUF;
	if ( (n=port_readline(pipefd[0],readprin,MAXLINE)) <0 ) {
		CLEARBUF;
		logerr(2,"Could not read from pipe");
		return( -1 );
	} else {
		if ( n == 0 )	return n;
		log(3,"I have read from pipe <%s>", readprin);
	}

/*
	if UNSOLDEV( getcrstr(readprin,1) ) {
		CLEARBUF;
		return n;
	}
*/
	if UNSOLCODE( getcrstr(readprin,2) ) {
		int i;

		cd		= get_num( getcrstr(readprin,2) );
		if ( inptr )
			 *inptr		= cd;
		CLEARBUF;
		if ( (i=check_code( cd, stderr )) == FILTRETRY ) {
			log(2,"Unsolicited problem with code =%d", cd);
			wait_printer(i);
		}
		return n;
	}
	if JOBSTART( getcrstr(readprin,3) ) {
		job_started	= TRUE;
		return n;
	}
	if JOBEND( getcrstr(readprin,3) ) {
		job_ended	= TRUE;
		return n;
	}
	return n;

}

/*
 * The passed argument is supposed to be a string starting off with any
 * pattern (such as "PAGES=" or "CODE=") and followed by a number.
 * The number is returned as this is the one of major concern.
 */
get_num(s)
char *s;
{
	int i;

	if (!s) return(0);

	while ( *s && (!isdigit(*s)) )	s++ ;
	if ( *s )	sscanf(s,"%d",&i);	
		else  i=0;

	return( i );
}

/*
 * The child attends the printer port and when data turn up it
 * grabs it. Data is passed through a pipe to the father.
 */
forkport()
{

	void childdied(), slay();
	int readpipe();

	if ( pipe(pipefd) < 0 ) {
		logerr(1,"Cannot set pipe");
		fexit(FILTABORT);
	}

	monitpid = fork();
	log(1,"Right after fork with monitpid=%d", monitpid);
	switch (monitpid) {
		case -1:
			logerr(1,"Cannot fork process");
			fexit(FILTABORT);
			break;
		case 0:				/* child */
			log(4,"Child with pid=%d",getpid());
			close(pipefd[0]);	/* Child writes on p[1] */
			log(4,"Trying to set the signals");
			if ( signal(SIGTERM, slay) < 0) {
				logerr(2,"Signal cannot be set");
				fexit(FILTSUCC);
			}
			if ( signal(SIGPIPE, slay) < 0) {
				logerr(2,"Signal cannot be set");
				fexit(FILTSUCC);
			}
			if ( signal(SIGBUS,slay) < 0) {
				logerr(2,"Signal cannot be set");
				fexit(FILTSUCC);
			}
			if ( signal(SIGSEGV,slay) < 0) {
				logerr(2,"Signal cannot be set");
				fexit(FILTSUCC);
			}
			if ( signal(SIGQUIT, slay) < 0) {
				logerr(2,"Signal cannot be set");
				fexit(FILTSUCC);
			}
			if ( signal(SIGINT, slay) < 0) {
				logerr(2,"Signal cannot be set");
				fexit(FILTSUCC);
			}
			if ( signal(SIGHUP, slay) < 0) {
				logerr(2,"Signal cannot be set");
				fexit(FILTSUCC);
			}
			log(4,"Signals have been set ok");
			monitorport();		/* Infinite loop for reading */
			break;
		default:			/* father */
			log(2,"Father with pid=%d",getpid());
			if ( close(pipefd[1])<0 )
				logerr("Cannot close pipe");	
				/* Father reads from p[0] */
			else	log(1,"After closing pipe in father");
			return;
	}
}

/*
 * This is the process to monitor for ever the port attached to the printer.
 * When it reads something it sends it through the pipe to its father.
 */
monitorport()
{
	char buf[MAXLINE+1];
	int rdcnt;


	log(3,"Monitoring the printer port");

	for( ; ; ) {
#ifdef SVR4
		memset(buf, 0, MAXLINE);
#else
		bzero(buf,MAXLINE);
#endif
		rdcnt=port_readline(STDOUT, buf, MAXLINE);

		if (rdcnt == -1)	{
			logerr(3,"Cannot read from port");
			continue;
		}

		if (rdcnt == 0)	continue;

		log(3,"I have read <%s> from printer",buf);

		if (writecn(pipefd[1],buf,strlen(buf))<0) {
			logerr(3,"...but cannot write it on pipe");
			fexit(FILTRETRY);
		}
	}/* for loop */
}

void slay(sig)
int sig;
{
	log(4,"Signal %u received", sig);
	fexit(FILTSUCC);
}

void nochildren()
{
	log(4,"SIGCHLD received by father");
	if ( job_ended == TRUE )
		fexit(FILTSUCC);
	else {
		log(3,"Calling cleanup...");
		cleanup(SIGCHLD);
		fexit(FILTRETRY);
	}
}

void pr_late()
{
	log(5,"In pr_late");
	alrmflag = 1;
}

pr_query(fd, n)
int  fd, n;
{
	short i, len ;

	for (i=0; i<n; i++) {
		len = strlen(query[i]);
		if (  writecn(fd, query[i], len) != len  )	{
			logerr(1,"Cannot write query string <%s>", query[i]);
			fexit(FILTABORT);
		} else
			log(3,"Writing query string <%s>", query[i]);
	}
}

/*
 * In the PJL manual, Chapter 9, Section: Determining if printer status
 * readback is available it states that if the application does not
 * receive a response within three seconds, then the application can 
 * assume printer status readback is currently unavailable.
 */
pr_synch(fd)
int fd;
{
	int synccont;
	int   synchronised = FALSE, blmask, fftries = 0;
#ifdef SVR4
	char buf[MAXLINE+1];
#else
	char *sprintf(), buf[MAXLINE+1];
#endif
	void (*signal())(), (*f)(), pr_late();

	log(2,"Synchronisation attempt");

	query[0]="\033%-12345X@PJL\r\n";
#ifdef SVR4
	sprintf(buf,"@PJL ECHO %d\r\n",getpid());
	query[1]=buf;
#else
	query[1]=sprintf(buf,"@PJL ECHO %d\r\n",getpid());
#endif
	pr_query(fd, 2);

	f=signal(SIGALRM,pr_late);
	alrmflag=0;
	alarm(60);

	do {
		log(4, "fftries=%d", ++fftries);
		if ( readpipe((char *)0) <= 0 )	continue;

		if ( !strncmp(readprin,buf,strlen(buf)-2) ) {
			synchronised = TRUE;
			alarm(0);
			break;
		}

		if ( strstr(readprin, "@PJL ECHO") ) {
			log(2,"Synchro input from previous attempt <%s>", readprin);
		}

	} while ( !alrmflag );

	signal(SIGALRM,f);

	return ( synchronised );
}

pr_status(fd)
int fd;
{
	int st=0, fftries = 0;
	char *str;

	log(2,"Asking for printer status");

	query[0]="@PJL INFO STATUS\r\n";
	pr_query(fd, 1);

	signal(SIGALRM,pr_late);
	alrmflag=0;
	alarm(120);

	do {
		log(4, "fftries=%d", ++fftries);
		readpipe(&st);
		if ( st > 0 )	{
			alarm(0);
			return( st );
		}

	} while ( !alrmflag );

	signal(SIGALRM,SIG_DFL);
	return( st );
}

pr_timed(fd)
int fd;
{
#ifdef SVR4
	char buf[MAXLINE+1];
#else
	char *sprintf(), buf[MAXLINE+1];
#endif

#ifdef SVR4
	sprintf(buf,"@PJL USTATUS TIMED = %d \r\n", wait_time);
	query[0]=buf;
#else
	query[0]=sprintf(buf,"@PJL USTATUS TIMED = %d \r\n", wait_time);
#endif
	pr_query(fd, 1);
}

pr_ustatusoff(fd)
int fd;
{
	query[0]="@PJL USTATUSOFF\r\n";
	pr_query(fd, 1);
}

pr_ustatus(fd,pjlstr)
int fd;
char *pjlstr;
{
	query[0]=pjlstr;
	pr_query(fd, 1);
}

pr_startjob(fd)
int fd;
{
#ifdef SVR4
	char buf[MAXLINE+1];
#else
	char *sprintf(), buf[MAXLINE+1];
#endif

#ifdef SVR4
	sprintf(buf,"@PJL JOB %s", job_start);
	query[0] = buf;
#else
	query[0]=sprintf(buf,"@PJL JOB %s", job_start);
#endif
	pr_query(fd, 1);
}

pr_endjob(fd)
int fd;
{
#ifdef SVR4
	char buf[MAXLINE+1];
#else
	char *sprintf(), buf[MAXLINE+1];
#endif

	query[0]= UELPJL ;
#ifdef SVR4
        sprintf(buf,"@PJL EOJ %s", job_end);
        query[1] = buf;
#else
        query[1]=sprintf(buf,"@PJL EOJ %s", job_end);
#endif
	query[2]= UEL ;

	pr_query(fd, 3);
}

check_job_end()
{
	int fftries = 0;

	log(2,"Checking for job end");

	signal(SIGALRM,pr_late);
	alrmflag=0;
	alarm(60);

	do {
		log(4, "fftries=%d", ++fftries);
		readpipe((char *)0);

		if ( job_ended == TRUE ) {	
			alarm(0);
			log(2,"Job ended successfully");
			return;
		} 
	} while ( !alrmflag );

	log(1,"Job ended unsuccessfully");
	fexit(FILTRETRY);
}

getpjlargs()
{
	sprintf(job_start,"NAME=\"Start of CTI-Print Job: %d\"\r\n",getpid());
	sprintf(job_end,"NAME=\"End of CTI-Print Job: %d\"\r\n", getpid() );
}

fexit(i)
int i;
{
	log(1,"Exit value = %d", i);
	if ( monitpid > 0 )	{
		int chld;

		log(3,"Father murders %d", monitpid);
		if ( kill(monitpid,SIGTERM) < 0 )
			logerr(3,"...but killing unsuccessful");
		chld = wait((int *)0);
		log(3,"wait returned %d", chld);
	}
	exit(i);
}

check_code( cd, fp )
int cd;
FILE *fp;
{
	log(5,"In check_code (status %d)",cd);

	if ( cd == 10001 ) {
		log(2,"Printer should be OK (status %d)",cd);
		return(FILTSUCC);
	}

	if ( (cd == 10000) || (cd == 35078) ) {
		log(2,"Printer in Powersave (status %d)",cd);
		wake_up();
		return(FILTRETRY);
	}

	if ( cd == 10002 ) {
		log(2,"Printer should be offline (status %d)",cd);
		return(FILTRETRY);
	}

	if ( cd == 10003 ) {
		log(2,"Printer warming up (status %d)",cd);
		return(FILTRETRY);
	}

	if ( cd == 10006 ) {
		log(2,"Toner Low (status %d)",cd);
		return(FILTSUCC);
	}

	if ( cd <= 10099 ) {
		log(2,"$$$ $$$ $$$ (status %d)",cd);
		return(FILTRETRY);
	}

	if ( cd <= 11499 ) {
		log(2,"Background paper loading (status %d)",cd);
		return(FILTSUCC);
	}

	if ( (cd > 20000) && (cd <= 20030) ) {
		log(2,"PJL syntax error (status %d)",cd);
		return(FILTSUCC);
	}

	if ( cd == 25002 )  {
		log(2,"PJL prefix missing (status %d)",cd);
		return(FILTSUCC);
	}

	if ( ( cd > 25000 ) && ( cd < 25999 ) )  {
		log(2,"PJL syntax warning (status %d)",cd);
		return(FILTSUCC);
	}

	if ( (cd >= 20001) && (cd <= 35999) ) {
		log(2,"$$$ $$$ $$$ (status %d)",cd);
		fexit(FILTABORT);
	}

	if ( cd == 40021 ) {
		log(2,"Printer open (status %d)",cd);
		return(FILTRETRY);
	}

	if ( cd == 40022 ) {
		log(2,"Paper jam (status %d)",cd);
		return(FILTRETRY);
	}

	if ( cd == 40079 ) {
		log(2,"Printer manually taken offline (status %d)",cd);
		return(FILTRETRY);
	}

	if ( (cd >= 41001) && (cd <= 41999) ) {
		log(2,"Foreground paper loading (status %d)",cd);
		return(FILTRETRY);	/* Needs paper */
	}

	log(2,"Unknown reason");
	return(FILTRETRY);
}

void wait_printer(i)
int i;
{
	if ( retries == -10 )	{
		log(2,"Let's sleep for %d secs", wait_time);
		sleep( wait_time );
		return;
	}

	if ( retries-- > 0 )	{
		log(2,"Let's sleep for %d secs", wait_time);
		sleep( wait_time );
	} else {
		log(2,"Can not sleep anymore");
		cleanup(SIGTERM);
		fexit(i);
	}
}

write_check(fd,str,msg,flg)
int  	 fd,   flg;
char	*str, *msg;
{
	int len, logerr(), logerr_die(), (*f)();

	len = strlen(str);
	if ( flg==DIE )
		f = logerr_die;
	else
		f = logerr;

	if ( writecn(fd, str, len) != len )		(*f)(3,msg);
}

get_retries()
{
	char *s;
	int r;

	s = getenv("RETRIES") ;
	if ( s ) {
		sscanf(s,"%d", &r);
		retries = r;
		if ( retries == 0 )	retries = -10 ;
	}

	s = getenv("SLEEPTIME") ;
	if ( s ) {
		sscanf(s,"%d", &r);
		wait_time = r;
	}
}

char *getcrstr(s, i)
char *s;
int i;
{
	int k=0, m, start;
	char *s1=(char *)0, *s2, *ptr=s;

	while (*ptr) 
		if ( *ptr++ == '\r') 	k++ ;	

	if ( i > k )	return(s);

	ptr = s ;
	start = 0;
	for ( m=0 ; m < strlen(s) ; m++ ) {
		if ( ptr[m] == '\015' ) {
			i-- ;
			if ( i == 0 )	return( &s[start] );
			else start = m + 2;
		}
	}

	return( s );
}

/*
 *
 */
