/*
 ===========================================================================
 =                                                                         =
 =  (C) Copyright 1995, Computer Technology Institute (CTI)                =
 =                                                                         =
 =  Permission to use, copy, modify and distribute this software and       =
 =  its documentation for non-commercial use and without fee is hereby     =
 =  granted, provided that the above copyright notice appear in all        =
 =  copies and that both that copyright notice and this permission         =
 =  notice and warranty disclaimer appear in supporting documentation,     =
 =  and that the name of Computer Technology Institute (CTI) or any of     =
 =  its entities not be used in advertising or publicity pertaining to     =
 =  distribution of the software and its documentation without specific    =
 =  prior permission.                                                      =
 =                                                                         =
 =  Computer Technology Institute (CTI) disclaims all warranties with      =
 =  regard to this software and makes no representations about the         =
 =  suitability of this software and its documentation for any purpose.    =
 =  It is provided "as is" without expressed or implied warranty.          =
 =                                                                         =
 ===========================================================================
 =                                                                         =
 =                        Project CTI-Print                                =
 =                                                                         =
 = File:                                                                   =
 =   log.c                                                                 =
 =                                                                         =
 = Synopsis:                                                               =
 =   This file contains subsidiary functions for error logging             =
 =                                                                         =
 = Author:                                                                 =
 =   Panos Dimakopoulos, Systems Programmer,                               =
 =   Computer Technology Institute,                                        =
 =   Division of Computing Facilities,                                     =
 =   P.O. Box 1122,                                                        =
 =   261 10  Patras,                                                       =
 =   Greece                                                                =
 =   (e-mail: dimakop@cti.gr)                                              =
 =   Tel: +30 61 992061                                                    =
 =   Fax: +30 61 993973                                                    =
 =                                                                         =
 ===========================================================================
*/


/****************************************************************************
 * Modification History:
 * 	Revision 1.1	95/01/12	11:45:37
 *		 Creation.
 * 	Revision 1.2	95/02/10	14:22:46
 *		 Full CTI address.
 * 	Revision 1.3	95/02/14	14:04:11
 *		 New Copyright notice.
 *
 */

#include <stdio.h>
#include <errno.h>
#include <time.h>

static char idlog[] = "@(#) log.c 95/02/14 - V 1.3" ;
extern char *name;		/* name of filter */
extern int  debug;
extern int errorcode;

/*VARARGS1*/
log(lvl, msg, a1, a2, a3)
	char *msg;
{
	char *Time_str();

	if ( lvl > debug )	return;
	(void)fprintf(stderr, "At %s...\nfrom %s: ", Time_str(), name);
	(void)fprintf(stderr, msg, a1, a2, a3);
	(void)putc('\n', stderr);
	(void)fflush(stderr);
}


/*VARARGS1*/
fatal(lvl, msg, a1, a2, a3)
	char *msg;
{
	log(lvl, msg, a1, a2, a3);
	cleanup();
	exit(errorcode);
}


/*VARARGS1*/
logerr(lvl, msg, a1, a2, a3)
	char *msg;
{
	extern int errno, sys_nerr;
	extern char *sys_errlist[];
	int err = errno;
	char *Time_str();

	if ( lvl > debug )	return;
	(void)fprintf(stderr, "At %s...\nfrom %s: ", Time_str(), name);

	if (msg){
		(void)fprintf(stderr, msg, a1, a2, a3);
		(void)fprintf(stderr, "- " );
	}
	if( err < sys_nerr ){
		(void)fprintf(stderr,"%s", sys_errlist[err]);
	} else {
		(void)fprintf(stderr, "Unknown error %d", err);
	}
	(void)putc('\n', stderr);
	(void)fflush(stderr);
}

/*VARARGS1*/
logerr_die(lvl, msg, a1, a2, a3)
	char *msg;
{
	logerr(lvl, msg, a1, a2, a3);
	cleanup();
	exit(errorcode);
}

char *
Time_str()
{
	time_t tvec;		/* time */
	char *ctime();
	static char s[40];

	(void)time(&tvec);
	(void)strcpy(s,ctime(&tvec));
	s[strlen(s)-1] = 0;
	return( s );
}
