/*
 ===========================================================================
 =                                                                         =
 =  (C) Copyright 1995, Computer Technology Institute (CTI)                =
 =                                                                         =
 =  Permission to use, copy, modify and distribute this software and       =
 =  its documentation for non-commercial use and without fee is hereby     =
 =  granted, provided that the above copyright notice appear in all        =
 =  copies and that both that copyright notice and this permission         =
 =  notice and warranty disclaimer appear in supporting documentation,     =
 =  and that the name of Computer Technology Institute (CTI) or any of     =
 =  its entities not be used in advertising or publicity pertaining to     =
 =  distribution of the software and its documentation without specific    =
 =  prior permission.                                                      =
 =                                                                         =
 =  Computer Technology Institute (CTI) disclaims all warranties with      =
 =  regard to this software and makes no representations about the         =
 =  suitability of this software and its documentation for any purpose.    =
 =  It is provided "as is" without expressed or implied warranty.          =
 =                                                                         =
 ===========================================================================
 =                                                                         =
 =                        Project CTI-Print                                =
 =                                                                         =
 = File:                                                                   =
 =   bphpiii.c                                                             =
 =                                                                         =
 = Synopsis:                                                               =
 =   This is the main driver module which handles the printer interface.   =
 =                                                                         =
 = Author:                                                                 =
 =   Panos Dimakopoulos, Systems Programmer,                                =
 =   Computer Technology Institute,                                        =
 =   Division of Computing Facilities,                                     =
 =   P.O. Box 1122,                                                        =
 =   261 10  Patras,                                                       =
 =   Greece                                                                =
 =   (e-mail: dimakop@cti.gr)                                              =
 =   Tel: +30 61 992061                                                    =
 =   Fax: +30 61 993973                                                    =
 =                                                                         =
 ===========================================================================
*/


/****************************************************************************
 * Modification History:
 * 	Revision 1.1	94/09/29	16:21:57
 *		 Creation.
 * 	Revision 1.2	94/12/08	12:27:28
 *		 The count of nulls to be sent is specified automatically
 *		 from the MODEL env variable which can be set to III or IIID.
 * 	Revision 1.3	95/01/24	19:39:21
 *		 The sleep time before the resetprinter thought not to be
 *		 necessary. However this series has great problems with
 *		 the switching.
 * 	Revision 1.4	95/01/30	17:42:13
 *		 Copyright label and file name's size changed.
 * 	Revision 1.5	95/01/30	18:51:10
 *		 Level added to log functions.
 * 	Revision 1.6	95/02/02	13:44:19
 *		 -B"<banner page title>" flag added.
 * 	Revision 1.7	95/02/09	20:22:40
 *		 -B"<banner page title>" flag can have a \n embedded to be
 *		 printed underneath the Institution's title.
 * 	Revision 1.7	95/02/10	18:02:20
 *		 Full CTI address.
 * 	Revision 1.8	95/02/14	13:09:32
 *		 New Copyright notice.
 * 	Revision 1.9	95/02/14	19:14:47
 *		 1994 Copyright becomes 1995.
 * 	Revision 1.10	95/03/03	15:56:11
 *		 Fixed bug in get_model where IIISi was ignored.
 * 	Revision 1.11	95/05/30	18:09:56
 *		 Version 1.1 initiated.
 * 	Revision 1.12	95/08/23	17:48:52
 *		 username, seq_number and date defined.
 * 	Revision 1.13	95/08/24	16:35:53
 *		 Version 1.2 initiated. Porting to Solaris.
 *
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/file.h>
#include <stdlib.h>
#include <signal.h>
#include <string.h>
#include <hpiii.h>
#ifdef SVR4
#include <unistd.h>
#endif

static char idallf[] = "@(#) Computer Technology Institute, Project CTI-Print";
static char idhpiii[] = "@(#)    Banner Filter for HP LaserJet III, V 1.2 \n" ;
static char idbphpiii[] = "@(#) bphpiii.c 95/08/24 - V 1.13" ;

/* 
 * The following variables are set by the arguments passed to the filter
 * from the caller (lpd)
 */
	char	*zopts, 
		*class, 
		*job, 
		*login, 
		*accntname, 
		*bantitle = "Computer Technology Institute\n\
Division of Computing Facilities", 
		*host, 
		*accntfile, 
		*format,
		*printer, 
		*seq_number, 
		*username, 
		*date, 
		*filename, 
		*pr_commnt;
	int	width, 
		length, 
		xwidth, 
		ylength,
		debug, 
		literal, 
		indent;

int prmsline;			/* line of printer_perms to change  */
int errorcode = FILTABORT;	/* value by which the errlog funcs exit */
int mode = 0;			/* PCL or Postscript */
char *name;			/* name of filter */
int npages;			/* number of pages */
int cartridge;
int model;			/* printer model */
char bline[250];

main( argc, argv )
	int argc;
	char **argv;
{
	int i;
	void cleanup();

	debug = 2;
	getargs( argc, argv );
	/*
	 * Turn off SIGPIPE
	 */
	(void)signal( SIGPIPE, SIG_IGN );
	(void)signal( SIGINT, cleanup );
	(void)signal( SIGTERM, cleanup );

	get_model();
	get_retries();
	baninit();
	banner( );

	npages = 1;	/* Just the header page */
	doaccnt();
	updaccnt(npages);
	exit(FILTSUCC);
}

/*
 *	doaccnt()
 *	writes the accounting information to the accounting file
 *  This has the format: user host printer pages format date
 */
doaccnt()
{
	time_t t, time();
	char *ctime();
	FILE *f;

	t = time((time_t *)0);
	if(accntfile && access(accntfile, W_OK) >= 0 &&
	    (f = fopen(accntfile, "a" )) != NULL) {
		fprintf(f,"%s\t%s\t%s\t%s\t%7d\t%s\t%s",
			login? login: "NULL", 
			host? host: "NULL", 
			printer? printer: "NULL", 
			filename? filename: "NULL",
			npages,
			format? format: "NULL", 
			seq_number ? seq_number : "XXX",
			ctime(&t));
		(void)fclose(f);
	}
}

void cleanup() 
{
	writecn(STDOUT,"\003\004", strlen("\003\004"));  /* ctrl-C, EOF */
	writecn(STDOUT,PCLRESETSTR, strlen(PCLRESETSTR));
	log(4,"Signal received: Job Aborted before it finishes");
	fexit(FILTABORT);
}


/******************************************************************************
 *	PCL Cover Description strings
 */
static char pcl_on[]	= {
	"\033E\033%-12345X"
};

static char pclhead10[] = {
	"\033*p30x50Y\033*c1800a100b45g2P"
} ;

static char pclhead12[] = {
	"\033(0U\033(s1p25v1s0b5T\033*p100x350Y%s"
} ;

static char pclhead13[] = {
	"\033(0U\033(s1p18v1s0b5T\033*p100x450Y%s"
} ;

static char pclhead2[] 	= {
	"\033*p30x560Y\033*c1800a100b25g2P\033(8U\033(s1p24v0s0b4148T" 
} ;

static char pclhead3[] 	= {
	"\033*p500x2900Y%40s\033(0N\033(s1p12v0s0b4T\033*p50x3200Y\251 \
\033(s9V\033*p80x3186Y \
1995, Computer Technology Institute (CTI)\033*p100x800Y\033(s18v3BJob:\033(s0B"
} ;


static char pclhead4[] 	= {
	"\033&dD %s \033&d@"
} ;

static char pclhead5[] 	= {
	" %s \033(s1Sfrom node:\033(s0S %s"
};

static char pclhead6[] 	= { 
	"\033*p300x1200Y\033(s56V%s\033*p100x1600Y\033(s24v3BFile:\033(s24v0B \
%s\033*p100x2000Y\033(s18v3BDate:\033(s0B %s"
} ;

baninit()
{
	has_cartridge();
	if (cartridge == ON)
		setpcl_on();
	else
		pclresetprinter();
}

has_cartridge()
{
	char *s;

	s = getenv("CARTRIDGE") ;
	if ( !s )	cartridge = OFF ;
	if ( ! strcmp(s,"ON") ) cartridge = ON;
		else {
			cartridge = OFF;
		}
}

banner( )
{
	char *Time_str(), *s2;


	sprintf(bline,"%s",pcl_on);
	Out_line();
	sprintf(bline,"%s",pclhead10);
	Out_line();
	if ( (s2=strchr(bantitle, '\n')) != NULL ) {
		*s2 = '\0' ;
		s2++ ;
		sprintf(bline,pclhead13,s2);
		Out_line();
	}
	sprintf(bline,pclhead12,bantitle);
	Out_line();
	sprintf(bline,"%s",pclhead2);
	Out_line();
	sprintf(bline,pclhead3,pr_commnt);
	Out_line();
	sprintf(bline,pclhead5,job,host) ;
	Out_line();
	sprintf(bline,pclhead6,login,filename,Time_str()) ;
	Out_line();

	pclresetprinter();
}

/***************************************************************************
 * Out_line( char* buf )
 * write the buffer out to the file descriptor.
 ***************************************************************************/
Out_line()
{
	int i;
	i = strlen(bline);

	if (writecn( fileno(stdout), bline, i ) != i)
			logerr(3,"Out_line: <%s> write failed\n", bline );
}

int writecn(fd,ptr,nbytes)
register int fd;
register char *ptr;
int nbytes;
{
	int nleft, nwritten;
	void wait_printer();
	void (*f)();
	
	nleft = nbytes;
	while (nleft > 0) {
		f=signal(SIGALRM,wait_printer);
		alarm(4);
		nwritten = write (fd, ptr, nleft);
		alarm(0);
		signal(SIGALRM,f);
		if (nwritten == 0) {
			logerr(4,"writecn wrote nothing");
			continue ;
		}
		if (nwritten < 0) {
			logerr(4,"writecn failure");
			fexit(FILTRETRY);
		}
		nleft -= nwritten;
		ptr += nwritten;
	}
	return(nbytes - nleft);
}

pclresetprinter()
{
	if ( writecn(STDOUT,PCLRESETSTR,strlen(PCLRESETSTR)) != strlen(PCLRESETSTR) )
		logerr(3,"Cannot pcl reset printer");
}

static char null_str[]={"\000\000\000\000\000\000\000\000"};
#define NULL_LEN	8

setpcl_on()
{
	int cou, counulls=300, slptm=10;

	if (model == IIID)	{ counulls = 3000; slptm = 20; }
	if (model != IIISi)	sleep(10);
	if ( writecn(STDOUT,EOFSTR,strlen(EOFSTR)) != strlen(EOFSTR) )
		logerr(3,"Cannot send EOF");
	if (model != IIISi)	sleep(10);
	resetprinter();
	for (cou=0;cou<counulls;cou++)
		if ( writecn(STDOUT,null_str,NULL_LEN) != NULL_LEN )
			logerr_die(1,"Cannot send nulls");
	if ( writecn(STDOUT,PCLON,strlen(PCLON)) != strlen(PCLON) )
		logerr_die(1,"Cannot set PCL mode");
	sleep(slptm);
}

resetprinter()
{
	if ( cartridge == OFF ) {
		pclresetprinter();
		return;
	}

	if ( writecn(STDOUT,UEL,strlen(UEL)) != strlen(UEL) )
		logerr(3,"Cannot UEL reset printer");
	if ( writecn(STDOUT,UEL,strlen(UEL)) != strlen(UEL) )
		logerr(3,"Cannot UEL reset printer");
}

get_model()
{
	char *s;

	s = getenv("MODEL");
	if (s) {
		if ( ! strcmp(s, "III") )	model = III ;
		if ( ! strcmp(s, "IIID") )	model = IIID ;
		if ( ! strcmp(s, "IIISi") )	model = IIISi ;
	}
}
