/*
 ===========================================================================
 =                                                                         =
 =  (C) Copyright 1995, Computer Technology Institute (CTI)                =
 =                                                                         =
 =  Permission to use, copy, modify and distribute this software and       =
 =  its documentation for non-commercial use and without fee is hereby     =
 =  granted, provided that the above copyright notice appear in all        =
 =  copies and that both that copyright notice and this permission         =
 =  notice and warranty disclaimer appear in supporting documentation,     =
 =  and that the name of Computer Technology Institute (CTI) or any of     =
 =  its entities not be used in advertising or publicity pertaining to     =
 =  distribution of the software and its documentation without specific    =
 =  prior permission.                                                      =
 =                                                                         =
 =  Computer Technology Institute (CTI) disclaims all warranties with      =
 =  regard to this software and makes no representations about the         =
 =  suitability of this software and its documentation for any purpose.    =
 =  It is provided "as is" without expressed or implied warranty.          =
 =                                                                         =
 ===========================================================================
 =                                                                         =
 =                        Project CTI-Print                                =
 =                                                                         =
 = File:                                                                   =
 =   fontsfile.c                                                           =
 =                                                                         =
 = Synopsis:                                                               =
 =   In this module is the code which scans file MACHINESFONTS to find     =
 =   what font is to be downloaded to the printer depending on the sender  =
 =   machine. A sample of such a file can be as follows:                   =
 =                                                                         =
 = 		hermes.cti.gr		dir1            file1              =
 = 		kosinthos.cti.gr	dir1            file2              =
 = 		atlas.cti.gr		dir2            file3              =
 = 		afrodite.cti.gr		                                   =
 =              # If the second or third arg is missing default font is    =
 =              # selected.                                                =
 = 		# A hash at the beginning of the line is for comments.     =
 =                                                                         =
 = Author:                                                                 =
 =   Panos Dimakopoulos, Systems Programmer,                               =
 =   Computer Technology Institute,                                        =
 =   Division of Computing Facilities,                                     =
 =   P.O. Box 1122,                                                        =
 =   261 10  Patras,                                                       =
 =   Greece                                                                =
 =   (e-mail: dimakop@cti.gr)                                              =
 =   Tel: +30 61 992061                                                    =
 =   Fax: +30 61 993973                                                    =
 =                                                                         =
 ===========================================================================
*/


/****************************************************************************
 * Modification History:
 * 	Revision 1.1	94/10/01	21:31:21
 *		 Creation.
 * 	Revision 1.2	95/01/30	19:16:26
 *		 Level added to log functions.
 * 	Revision 1.3	95/02/10	18:35:06
 *		 Full CTI adddress.
 * 	Revision 1.4	95/02/14	13:20:25
 *		 New Copyright notice.
 *
 */

#include <stdio.h>
#include <fcntl.h>
#include <hpiii.h>

static char idfonts[] = "@(#) fontsfile.c 95/02/14 - V 1.4" ;

FILE *fnt_open( fname )
char *fname;
{
	FILE *fp;
	char Host[150+1], filename[MAXLINE];

	strcpy(filename,fname);

	if ( gethostname(Host,150) < 0 ) {
		logerr(1,"Cannot find my machine name");
		return ( (FILE *)NULL );
	} else
		strcat(filename,Host);

	if ( (fp=fopen(filename, "r")) == NULL ) {
		logerr(1,"Cannot open machines fonts file <%s>", fname);
		return ( (FILE *)NULL );
	} else
		return( fp );
}

fnt_close( fp )
FILE *fp;
{
	fclose(fp);
}

char *fnt_rdline( fntfp )
FILE *fntfp;
{
	static char s[MAXLINE+1];

	if ( fgets(s, MAXLINE, fntfp)==NULL )	return( (char *)NULL );
	else					return( s );
}

char *fnt_rd( line, mchn )
char *line;
char *mchn;
{
	char  machine[MAXLINE+1], fontdir[MAXLINE+1], fontfile[MAXLINE+1];
	static char fontpath[MAXLINE+1];
	short n;

	n = sscanf(line, "%s%s%s", machine, fontdir, fontfile);
	if (n != 3) {
		return( (char *)0 );
	}
	if (! strcmp(machine,mchn) ) {
		sprintf(fontpath,"%s/%s", fontdir, fontfile);
		log(2,"Font path selected: <%s>", fontpath);
		return( fontpath );
	}
	else
		return( (char *)NULL );
}

char *fnt_scan( mchn )
char *mchn;
{
	FILE *fontfp;
	char *fnt_rdline(), *fnt_rd(), *s;

	fontfp = fnt_open( MACHINESFONTS );
	if ( ! fontfp ) {
		fnt_close( fontfp );
		return( (char *)NULL );
	}

	while( s=fnt_rdline( fontfp ) )
	{
		if ( *s == '#' )	continue;
		if (s=fnt_rd(s,mchn))	break;
	}

	fnt_close( fontfp );
	return( s );
}
