/*==========================================================================
 =                                                                         =
 =                        Project CTI-Print                                =
 =                                                                         =
 = Author:                                                                 =
 =   Panos Dimakopoulos, Systems Programmer,                               =
 =   Computer Technology Institute,                                        =
 =   Division of Computing Facilities,                                     =
 =   P.O. Box 1122,                                                        =
 =   261 10  Patras,                                                       =
 =   Greece                                                                =
 =   (e-mail: dimakop@cti.gr)                                              =
 =   Tel: +30 61 992061                                                    =
 =   Fax: +30 61 993973                                                    =
 =                                                                         =
 = Created by Patrick Powell  <papowell@sdsu.edu>                          =
 =   for LPRng software Sat Aug 26 06:54:25 PDT 1995                       =
 ==========================================================================*/


/*==========================================================================
  Version CTI-LPRng-1.0
  =========================================================================*/


/*
 * $Id: do_monitor.c,v 1.3 1995/11/06 03:16:43 papowell Exp $
 */

#include "portable.h"
#include "hp4.h"
#include "common.h"

/*
 * set up the child process to do the monitoring and synchronizaiton
 */


void pr_ustatus(fd,pjlstr)
int fd;
char *pjlstr;
{
	int i = 0;
	query[i++]=pjlstr;
	pr_query(fd, i);
}

void do_monitor()
{
	int printer_code;

	if ( pipe(pipefd) < 0 ) {
		logerr(1,"Cannot set pipe");
		fexit(FILTABORT);
	}
	monitpid = fork();

	log(4,"Right after fork with monitpid=%d", monitpid);
	switch (monitpid) {
		case -1:
			logerr(1,"Cannot fork process");
			fexit(FILTABORT);
			break;
		case 0:				/* child */
			log(4,"Child with pid=%d",getpid());
			/* Child writes on p[1] */
			if( close(pipefd[0]) < 0 ){
				logerr(2,"Cannot close pipe");	
			}
			log(4,"Trying to set the signals");
			(void)signal(SIGTERM, slay);
			(void)signal(SIGQUIT, slay);
			(void)signal(SIGINT, slay);
			(void)signal(SIGHUP, slay);
			log(4,"Signals have been set ok");
			monitorport();		/* Infinite loop for reading */
			exit(0);
			break;

		default:			/* father */
			log(4,"Father pid=%d, child %d",getpid(), monitpid);
			if ( close(pipefd[1])<0 ){
				logerr(2,"Cannot close pipe");	
				/* Father reads from p[0] */
			} else {
				log(4,"After closing pipe in father");
			}
			break;
	}

   	/*
	* Try to synchronise with printer; use 60 secs as good default
	*/
	if( pr_synch(STDOUT, 60 )==FALSE ) {
		log(2,"Failed to synchronise with printer");
		fexit(FILTRETRY);
	} else {
		log(2,"Synchronised with printer");
	}

   /*
	* Check printer status
	*/

	printer_ok = FALSE;
	do {
		pr_ustatus(STDOUT, INFOSTATUS);	/* job status on */
		(void)readpipe( &printer_code,3);
		if( printer_code <= 0 ) continue;
		switch( printer_code ){
			case 10001:		/* Printer ready */
				printer_ok = TRUE;
				break;
			case 10003:		/* Printer warming up */
				printer_ok = TRUE;
				break;
			case 10000:		/* Printer in powersave mode */
			case 35078:		
				printer_ok = TRUE;
				break;
			case 10006:		/* Printer ready */
				printer_ok = TRUE;
				break;
			default:
				sleep(1);
				break;
		}
	} while	( printer_ok==FALSE );

	/* get initial page count */

	if( pagecount ){
		initialpagecount = pr_pagecount(STDOUT);
	}
	log(2,"Initial page count %d", initialpagecount );
	doaccnt(1);

	/* pr_ustatus(STDOUT,"@PJL USTATUS DEVICE = VERBOSE\r\n"); */

}
