/*==========================================================================
 =                                                                         =
 =                        Project CTI-Print                                =
 =                                                                         =
 = Author:                                                                 =
 =   Panos Dimakopoulos, Systems Programmer,                               =
 =   Computer Technology Institute,                                        =
 =   Division of Computing Facilities,                                     =
 =   P.O. Box 1122,                                                        =
 =   261 10  Patras,                                                       =
 =   Greece                                                                =
 =   (e-mail: dimakop@cti.gr)                                              =
 =   Tel: +30 61 992061                                                    =
 =   Fax: +30 61 993973                                                    =
 =                                                                         =
 = Created by Patrick Powell  <papowell@sdsu.edu>                          =
 =   for LPRng software Sat Aug 26 06:54:25 PDT 1995                       =
 ==========================================================================*/


/*==========================================================================
  Version CTI-LPRng-1.0
  =========================================================================*/


/*
 * $Id: doaccnt.c,v 1.4 1995/11/06 04:27:03 papowell Exp papowell $
 */

#include "portable.h"
#include "common.h"
#include "hp4.h"


/*
 *  doaccnt()
 *  writes the accounting information to the accounting file
 *  This has the format: user host printer pages format date
 */

static int argc;
static char *parms[20];
static char list[1024];

void inv( char *key, char *value )
{
	int len;
	len = strlen( list );
	if( value && *value ){
		if( key ){
			plp_snprintf( list+len, sizeof(list)-len, " %s'%s'", key, value );
		} else {
			plp_snprintf( list+len, sizeof(list)-len, " %s", value );
		}
		parms[argc++] = &list[len+1];
	}
	parms[argc] = 0;
}

void doaccnt(int start)
{
	int fd;
	int i, err, pid;
	uid_t child;
	int status;
	char pid_str[32];
	char totalpages[32];
	char pages[32];
	char *sh;
	struct stat statb;
	RETSIGTYPE (*olds)();


	/* safe use of sprintf */
	sprintf( pid_str, "%d", getpid() );

	/* we first set up the output line and arguments */
	sprintf( pages, "%d", npages );

	if( start ){
		strcpy( list, "start " );
	} else {
		strcpy( list, "end " );
	}
	argc = 0;
	sh = ACCNTSH;
	if( !start && sh && *sh ){
		if( stat( sh, &statb ) < 0 ){
			logerr(3,"Accounting script '%s' cannot stat- %s",
				sh, Errormsg(errno) );
			sh = 0;
		}
		if( sh && ! ( (statb.st_mode & 0001)
			|| ((statb.st_mode & 0100 ) && (statb.st_uid == getegid() ))
			|| ((statb.st_mode & 0010 ) && (statb.st_uid == geteuid() )) ) ){
			logerr(3,"Accounting script '%s' has bad exec perms or ownership",
				sh );
			sh = 0;
		}
	} else {
		sh = 0;
	}
	parms[argc++] = sh;
	if( !start ){
		sprintf( totalpages, "%d", npages-initialpagecount );
		inv( "-b", totalpages );
	}
	inv( "-q", pid_str );
	inv( "-k", controlfile );
	inv( "-n", login );
	inv( "-h", host );
	inv( "-P", printer );
	inv( "-p", pages );
	inv( "-F", format );
	inv( "-R", accntname );
	inv( "-t", Time_str() );
	
	/* this is probably the best way to do the output */
	if(accntfile && (fd = open(accntfile, O_WRONLY|O_APPEND )) >= 0) {
		i = strlen( list );
		list[i] = '\n';
		writecn(fd,list,i+1);
		(void)close(fd);
		list[i] = 0;
	}
	if( !start && sh && *sh ){
		inv( 0, accntfile );
		log(4,"accounting '%s %s'", parms[0], list );
		/* fix up parm list */
		for( i = 1; i < argc; ++i ){
			parms[i][-1] = 0;
		}
		parms[argc] = 0;
		/* turn off signal handling for children */
		olds = signal(SIGCHLD, SIG_DFL );
		child = fork();
		if( child == (uid_t)(-1) ) {
			(void)signal(SIGCHLD, olds );
			errorcode = FILTABORT;
			logerr_die(1,"Cannot fork to update accounting");
		} else if( child == 0 ){
			/* child - Exec the script to update pages */
			dup2(2,1);
			execve(sh,parms,Envp);
			errorcode = FILTABORT;
			logerr_die(1,"Cannot exec %s to update pages field",sh);
		} else {
			/* father - Wait for the update
			 * Child exits with 0 for Ok, 2 if signal
			 * terminated and 1 otherwise.
			 */
			while(1){
				status = 0;
				i = wait( &status );
				err = errno;
				log(4,"doaccnt: pid  %d, status 0x%x", pid, status );
				if( i == -1 ){
					errno = err;
					if( err != SIGCHLD ){
						logerr(1,"doaccnt: wait failed" );
						continue;
					} else {
						logerr(1,"doaccnt: no child" );
						break;
					}
				} else if( WIFEXITED(status) && WEXITSTATUS(status) == 0 ){
					break;
				}
				log(4,"Accounting process died, status 0x%x", status);
				fexit(FILTABORT);
			}
		}
		(void)signal(SIGCHLD, olds );
	}
}

