/*==========================================================================
 =                                                                         =
 =                        Project CTI-Print                                =
 =                                                                         =
 = Author:                                                                 =
 =   Panos Dimakopoulos, Systems Programmer,                               =
 =   Computer Technology Institute,                                        =
 =   Division of Computing Facilities,                                     =
 =   P.O. Box 1122,                                                        =
 =   261 10  Patras,                                                       =
 =   Greece                                                                =
 =   (e-mail: dimakop@cti.gr)                                              =
 =   Tel: +30 61 992061                                                    =
 =   Fax: +30 61 993973                                                    =
 =                                                                         =
 = Created by Patrick Powell  <papowell@sdsu.edu>                          =
 =   for LPRng software Sat Aug 26 06:54:25 PDT 1995                       =
 ==========================================================================*/


/*==========================================================================
  Version CTI-LPRng-1.0
  =========================================================================*/


/*
 * $Id: page_set.c,v 1.1 1995/09/02 15:12:45 papowell Exp $
 */

#include "portable.h"
#include "common.h"
#include "hp4.h"




void page_set()
{
	char s[150], *ptr;
	int len;
	int lnd = 0;
	int duplex = 0;
	int incr = 0;


	log(4,"page_set: starting" );
	write_check(STDOUT, CRLFSTR, "Cannot send CRLF string", 0);

	selectfont();

	if( len == 0 ) len = 60;
	if( width == 0 ) width = 80;

	ptr = PGSTR;

	if( zopts ){
		if ( strstr(zopts, "landscape")!=NULL ) {
			lnd = 1 ;
			width = 132 ;
			length = 41;
		}
		if( strstr(zopts, "duplex") !=NULL ){
			duplex = 1;
			ptr = PGSTRDPL;
			incr = 5;
		}
	}
	plp_snprintf(s, sizeof(s), ptr, lnd, width+incr, length);

	log(4,"page_set: lnd %d, width %d, length %d, sending '%s'",
		lnd, width, length, s );

	len = strlen(s);
	if ( writecn(STDOUT,s,len) != len ) {
		logerr(3,"Cannot send Control Page string");
	}

	if( lnd ){
		ptr = PITCHLND;
	} else if ( width==132 ){
		ptr = PITCH132;
	} else {
		ptr = PITCH80;
	}
	log(4,"page_set: pitch sending '%s'", ptr );
	len = strlen(ptr);
	if ( writecn(STDOUT,ptr,len) != len ){
		logerr(3,"Cannot send PITCH string");
	}

	if( duplex ){
		ptr = SHIFTDPL;
	} else {
		ptr = SHIFT;
	}
	log(4,"page_set: duplex %d sending '%s'", duplex, ptr );
	len = strlen(ptr);
	if( writecn(STDOUT,ptr,len) != len ){
		logerr(3,"Cannot send SHIFT string");
	}

	/*
	 * If line wrapping is required the WRAP env variable should be set to ON.
	 */
	if( wrap ){
		log(4,"page_set: wrap %d sending '%s'", wrap, WRAPON );
		write_check(STDOUT, WRAPON, "Cannot send WRAPON string", 0);
	}
}

