/*==========================================================================
 =                                                                         =
 =                        Project CTI-Print                                =
 =                                                                         =
 = Author:                                                                 =
 =   Panos Dimakopoulos, Systems Programmer,                               =
 =   Computer Technology Institute,                                        =
 =   Division of Computing Facilities,                                     =
 =   P.O. Box 1122,                                                        =
 =   261 10  Patras,                                                       =
 =   Greece                                                                =
 =   (e-mail: dimakop@cti.gr)                                              =
 =   Tel: +30 61 992061                                                    =
 =   Fax: +30 61 993973                                                    =
 =                                                                         =
 = Created by Patrick Powell  <papowell@sdsu.edu>                          =
 =   for LPRng software Sat Aug 26 06:54:25 PDT 1995                       =
 ==========================================================================*/


/*==========================================================================
  Version CTI-LPRng-1.0
  =========================================================================*/


/*
 * $Id: port_read.c,v 1.1 1995/09/02 15:12:46 papowell Exp $
 */

#include "portable.h"
#include "common.h"
#include "hp4.h"


/*
 * Read a line from a descriptor with timeout.
 *  Read the line one byte at a time,
 * looking for '\015' or '\014' (FF is the terminator for all responses
 * from the series 4 printers).  We store the FF in the buffer,
 * then follow it with a null (the same as fgets(3)).
 * We return the number of characters up to, but not including,
 * the null (the same as strlen(3)).
 */

int port_readline(fd, ptr, maxlen, timec )
int    fd;
char   *ptr;
int    maxlen;
int    timec;
{
    int                      n, rc, err;
    int                      c;
    struct timeval           timeout;
    fd_set                   readfds;

	log(4,"port_readline: fd %d, maxlen %d, timeout %d", fd, maxlen, timec );

    for( n = 0; n < maxlen; n++ ){
		do{
			FD_ZERO(&readfds);
			FD_SET( fd, &readfds);
			memset(&timeout, 0, sizeof(timeout) );
			if( timec >= 0 ){
				timeout.tv_sec = timec;
			} else {
				timeout.tv_sec = 1024*1024;	/* whacking great number */
			}
			if( n > 0 ){
				timeout.tv_sec = 1;
			}
			rc = select(fd+1, &readfds,(fd_set *)0,(fd_set *)0, &timeout);
			err = errno;
			log(8,"port_readline: select returned %d", rc );
			if( rc < 0 ){
				if( err==EINTR ){
					log(6,"Select received signal");
				} else {
					errno = err;
					logerr(1,"select error");
					fexit(FILTRETRY);
				}
			}
		} while( rc < 0 );
		/* we have something to read OR EOF */
		if( rc > 0 ){
			rc = read(fd, &ptr[n], 1);
			c = ptr[n];
			err = errno;
			log(8,"port_readline: read returned %d, char 0x%02x (%c)",
				rc, c, isprint(c)?c:'?'  );
		}
		if( rc <= 0 ){
			break;
		} else if( c == '\014' ){
			++n;
			break;
		}
    }
	/* handle the error conditions or EOF easily */
	ptr[n] = 0;
	if( n == 0 ){
		n = rc;
	}
	log(4,"port_readline: read %d, '%s'", n, ptr );
    return(n);
}
