/* lprps.c */

#ifndef lint
static char rcsid[] = "$Id: lprps.c,v 1.20 1993/02/22 12:44:26 jjc Exp $";
#endif

#include <stdio.h>
#include <ctype.h>
#ifdef SYSV
#include <string.h>
#define rindex   strrchr
#else
#include <strings.h>
#endif /* SYSV */
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/file.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/time.h>
#include <signal.h>
#include <syslog.h>
#include <errno.h>

#ifndef errno
extern int errno;
#endif

#ifdef __STDC__
#define VOLATILE volatile
#else
#define VOLATILE /* as nothing */
#endif

#define SENDMAIL "/usr/lib/sendmail"

#define EXIT_SUCCESS 0
#define EXIT_REPRINT 1
#define EXIT_THROW_AWAY 2

#define OBSIZE 1024
#define IBSIZE 1024

/* number of times to try to get the status from the printer at the
beginning of the job */
#define MAX_TRIES 1

char ctrl_d = '\004';
char ctrl_t = '\024';

char *malloc();

/* user's login name */
char *login = 0;
/* user's host */
char *host = 0;
/* name of accounting file */
char *accounting_file = 0;
/* stream to mailer process or log file */
FILE *mailfp = 0;
/* pid of the mailer process */
int mail_pid = -1;
/* set to non-zero on interrupt */
VOLATILE int interrupt_flag = 0;
/* set to non-zero on timeout */
VOLATILE int timeout_flag = 0;
/* if non-zero exit on timeout */
int exit_on_timeout = 0;
/* descriptor of printer device */
int psfd = 1;
/* number of ^D's received from printer */
int eof_count = 0;
/* if non-zero ignore all input from printer other than 004s */
int ignore_input = 0;
/* are we currently printing the users job */
int in_job = 0;
/* the contents of the status file at the beginning of the job */
char *status_file_contents = NULL;
/* name of the printer, or NULL if unknown */
char *printer_name = NULL;
/* non-zero if the contents of the status file are not status_file_contents */
int status_file_changed = 0;
/* non-zero if we need to ask the printer for its status, so that we can tell
when a printer error has been corrected */
int want_status = 0;
/* name of the status file */
char *status_filename = "/var/spool/lp1/status";
/* name of the job, or NULL if unknown */
char *job_name = NULL;
/* non-zero if errors from the printer should be mailed back to the user */
int mail_flag = 1;

enum {
    NORMAL,
    HAD_ONE_PERCENT,
    HAD_TWO_PERCENT,
    IN_MESSAGE,
    HAD_RIGHT_BRACKET,
    HAD_RIGHT_BRACKET_AND_PERCENT
} parse_state;

enum {
    INVALID,
    UNKNOWN,
    WARMING_UP,
    INITIALIZING,
    IDLE,
    BUSY,
    WAITING,
    PRINTING,
    PRINTER_ERROR
} status = INVALID;

int start_pagecount = -1;
int end_pagecount = -1;

struct {
    char *ptr;
    int count;
    char buf[OBSIZE];
} out = { out.buf, OBSIZE };

#define MBSIZE 1024

char message_buf[MBSIZE];
int message_buf_i;

/* last printer error message */
char error_buf[128];

static char pagecount_string[] = "(%%[ pagecount: ) print \
statusdict begin pagecount end 20 string cvs print \
( ]%%) print flush\n";

void send_file()
{
    int c;

    in_job = 1;
    parse_state = NORMAL;
    ignore_input = 0;
    while ((c = getchar()) != EOF)
	putchar(c);
}

int main(argc, argv)
    int argc;
    char **argv;
{
    char *prog;
    int i = 1;
    int Uflag = 0;

    if (argc > 0) {
	prog = rindex(argv[0], '/');
	if (prog)
	    prog++;
	else
	    prog = argv[0];
    }
    else
	prog = "lprps";
    openlog(prog, LOG_PID, LOG_LPR);

    for (i = 1; i < argc && argv[i][0] == '-'; i++)
	switch (argv[i][1]) {
	case 'c':
	case 'C':
	    break;
	case 'M':
	    /* don't mail errors back to user */
	    mail_flag = 0;
	    break;
	case 'U':
	    /* argument is not a real user (eg daemon) */
	    Uflag++;
	    break;
	case 'x':
	case 'y':
	case 'w':
	case 'l':
	case 'i':
	case 'F':
	    /* not interested in these */
	    break;
	case 'n':
	    login = &argv[i][2];
	    break;
	case 'h':
	    host = &argv[i][2];
	    break;
	    /* 'j', 'p', and 's' are non-standard */
	case 'J':
	    job_name = &argv[i][2];
	    break;
	case 'P':
	    printer_name = &argv[i][2];
	    break;
	}
    if (i < argc)
	accounting_file = argv[i];
    if (Uflag && mail_flag && login) {
	for (i = 1; i < argc && argv[i][0] == '-'; i++)
	    switch(argv[i][1]) {
	    case 'U':
		if (strcmp(argv[i] + 2, login) == 0)
		    mail_flag = 0;
		break;
	    case 'h':
	    case 'n':
	    case 'j':
	    case 'p':
	    case 's':
		i++;
		break;
	    }
    }
    send_file();
    exit(EXIT_SUCCESS);
    /*NOTREACHED*/
}
