<HTML><HEAD>
<TITLE>Page database source</TITLE></HEAD>
<BODY>
<PRE>
/*
 * Server stub that has routines that are called by server RPC
 * @(#)pgcntd.c 1.11    10/16/95
 */
#include <stdio.h>
#include <fcntl.h>          /* needed for opens */
#include <sys/types.h>
#include <syslog.h>
#include <errno.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <string.h>
#include "global.h"
#include "rsaref.h"
#include "common.h"

/*
 * Making these structures static is easier than
 * Passing the structure on down through the authentication routines
 */
static char encmsgholder[E_MSGSIZE];                /* buffer for received msg */
static char encsigholder[MAX_PEM_SIGNATURE_LEN];    /* buffer for received sig */
int lockfile;   

typedef struct _user_structure {
    char *user;
    char *authenticator;
    char *encmsg;
    int  msglen;
    char *encsig;
    int  siglen;
    int  delta;
} USERSTRUCT;

void LogFail(char *msg, int errno)
{
    openlog("pgcntd", LOG_CONS|LOG_PID, LOG_DAEMON);
    syslog(LOG_ERR, "%s - %d\n", msg, errno);
    closelog();
}
    
int *pages_set_1 (char **user)
{
}

PGSTRUCT *init_user (char *user)
{
    static int pages;
    static PGSTRUCT pgstruct, *pgp;
    datum dbkey;
    datum content;
    datum newdata;

    if ((dbfp = dbm_open(db_file, O_RDWR|O_CREAT, 0644)) == NULL) {
        LogFail("Couldn't open pagecount database", errno);
        exit(4);
    }
    pgstruct.used = 0;
    pgstruct.free = 100;
    pgstruct.bought = 0;
    newdata.dptr = (char *) &pgstruct;
    newdata.dsize = sizeof(pgstruct);
    dbkey.dptr = (char *) user;
    dbkey.dsize = strlen (user);
    content = dbm_fetch(dbfp, dbkey);
#ifdef DEBUG
    printf("Init user\n");
#endif
    if (content.dptr == NULL) {         /* return pgstruct as is */
        while (!excl_lock(lockfile, F_WRLCK))
            sleep(1);
        if (dbm_store(dbfp, dbkey, newdata, DBM_INSERT) < 0) {
            un_excl_lock(lockfile);
            LogFail("Failed on insertion of new user record", errno);
            exit(4);
        }
        pages = 100;
    } else  {                           /* return new pgstruct in dptr */
        pgp = (PGSTRUCT *) content.dptr;
        memcpy((char *) &pgstruct, (char *) pgp, sizeof(PGSTRUCT));
    }
    return (&pgstruct);
}

int pages_avail (char *user)
{
    static int pages;
    PGSTRUCT *pgp;

#ifdef DEBUG
    printf("pages avail\n");
#endif
    while (!excl_lock(lockfile, F_RDLCK))
        sleep(1);
    pgp = init_user(user);
    un_excl_lock(lockfile);
#   ifdef DEBUG
        printf("getpages: called by user %s\n", user);
#   endif
    pages = pgp->free + pgp->bought;
    dbm_close(dbfp);
    return (pages);
}

int doRSA(Msgstruct *userinfo)
{
    R_RSA_PUBLIC_KEY PublicKey;
    char encodedSig[MAX_PEM_SIGNATURE_LEN];
    int encodedSiglen;
    char encodedContent[200];
    int encodedContentLen;
    struct sockaddr_in *caller;
    int addrlen;
    Msgstruct upass;

    if (!ReadPubKey(userinfo->authenticator, &PublicKey)) {
        char buf[80];
        sprintf(buf, "Couldn't read public key for %-8.8s",
                            userinfo->authenticator);
        LogFail(buf, 0);
        exit(4);
    }
    encodedSiglen  = userinfo->siglen;
    memcpy(encodedSig, userinfo->encsig, encodedSiglen);
    encodedContentLen = userinfo->msglen;
    memcpy(encodedContent, userinfo->encmsg, encodedContentLen);
    
    /* Now setup a new structure to pass so that we don't munge 
    * this one for later */
    memcpy((char *) &upass, (char *) userinfo, sizeof(Msgstruct));
    upass.encsig = (char) NULL;
    upass.encmsg = (char) NULL;
    
    if (!VerifyBlock(&PublicKey, (char *) &upass, sizeof(Msgstruct), 
                encodedContent, encodedContentLen,
                encodedSig, encodedSiglen)) {
        return(FALSE);
    }

    return(TRUE);
}

int pages_delta (Msgstruct userinfo, char *cliname)
{
    static int pages;
    PGSTRUCT pgstruct, *pgp;
    datum dbkey;
    datum content;
    datum newdata;
    int remainder;
    int used, left, free, bought;
    register int delta;
    

    while (!excl_lock(lockfile, F_WRLCK))
        sleep(1);
    pgp = init_user(userinfo.user);
    memcpy((char *) &pgstruct, (char *) pgp, sizeof(pgstruct));
    pages = pgstruct.free + pgstruct.bought;
    delta = userinfo.delta;
#   ifdef DEBUG
        printf("deltapages: called by user %s - delta %d left %d\n", 
                userinfo.user, delta, pages);
#   endif
    newdata.dptr = (char *) &pgstruct;
    newdata.dsize = sizeof(pgstruct);
    dbkey.dptr = (char *) userinfo.user;
    dbkey.dsize = strlen (userinfo.user);
    /* debit/credit account */
    if ((remainder = pages + delta) >= 0) {     
        if (delta < 0) {        /* debit */
            pgstruct.used -= delta;
        } else {
            if(!doRSA(&userinfo)) {
                openlog("pgcntd", LOG_PID, LOG_AUTH);
                syslog(LOG_NOTICE, "page theft attempt from user %s, using %s@%s\n",
                        userinfo.user, userinfo.authenticator, cliname);
                un_excl_lock(lockfile);
                closelog();
                exit(2);
            }
        }
        pages = remainder;
        /* A positive delta, can be a credit or bought pages operation */
        if (delta > 0) {
            if (userinfo.operation == DELTA) {
                pgstruct.bought += delta;
            } else if (userinfo.operation == CREDIT) {
                pgstruct.free += delta;
            }
        } else {
            /* A negative delta */
            remainder = pgstruct.free += delta; /* subtract from free pgs 1st */
            /* no more free, substract from bought too */
            if (remainder < 0) {
                pgstruct.free = 0;
                pgstruct.bought += remainder;
            } else {            /* Enough free to cover page deficit */
                pgstruct.free = remainder;
            }
        }
    } else {    /* tried to print too much */
        pages = -1;
        un_excl_lock(lockfile);
        dbm_close(dbfp);
        return(pages);
    }
    if (dbm_store(dbfp, dbkey, newdata, DBM_REPLACE) < 0) {
        un_excl_lock(lockfile);
        LogFail("Failed on user record pagecount replacement attempt", errno);
        exit(4);
    }
    un_excl_lock(lockfile);
    dbm_close(dbfp);

    return (pages);
}

/*
 * Make sure all data is read from the stream
 */

void get_sock_data(int sock, char *msg, int msgsize)
{
    int rdtot, rdleft, rdsz;
    int reterr = -1;
    
    rdtot=0;
    rdleft = msgsize;
    do {
        rdsz = read(sock, msg+rdtot, rdleft);
        if (rdsz <= 0) {
#ifdef DEBUG
            perror("err reading from sock");
#endif
            write(sock, (char *) &reterr, sizeof(long));
            close(sock);
            exit(0);
        }
        rdleft -= rdsz;
        rdtot += rdsz;
    } while (rdtot < msgsize);
}

main()
{
    int sock;
    int childpid;                       /* fork off a handler */
    int newsock;                        /* new for socket */
    int ret_code;                       /* send back to other side */
    struct sockaddr_in cli_addr;        /* client calling */
    int clilen = sizeof(cli_addr);
    Msgstruct msg;                      /* message from client */
    char cliname[50];                   /* name of calling client */

    /* Change uid to admin and gid to rtfm - started as root*/
    setgid(172);
    setuid(16);
    setegid(172);

    lockfile = open("/tmp/pgcnt.lock", O_RDWR|O_CREAT, 0600);
    if (lockfile < 0) {
        perror("couldn't open lockfile");
        exit(1);
    }

    forkandsock(&sock);

    for ( ; ; ) {
        newsock = accept(sock, (struct sockaddr *) &cli_addr, &clilen);
        
        if (newsock < 0) {
            if (errno != 4)
                LogFail("Couldn't get new socket", errno);
            continue;
        }

#if !DEBUG
        if ((childpid = fork()) < 0)
            LogFail("couldn't fork handler", errno);
        else if (childpid == 0) {   /* as seen by child */
            close(sock);
#endif
        

            get_cliname(&cliname, &cli_addr, 50);
            get_sock_data(newsock, (char *) &msg, sizeof(Msgstruct));

            switch(msg.operation) {
                case AVAIL:
                    ret_code = pages_avail(msg.user);
                    break;
                case CREDIT:
                case DELTA:
                    if (msg.msglen > 0) {
                        get_sock_data(newsock, (char *) &encmsgholder, msg.msglen);
                        get_sock_data(newsock, (char *) &encsigholder, msg.siglen);
                        msg.encmsg = encmsgholder;
                        msg.encsig = encsigholder;

                    }
                    ret_code = pages_delta(msg, cliname);
                    break;
                default:
                    break;
            }

#ifdef DEBUG
            printf("returning %d\n", ret_code);
#endif
            write(newsock, (char *) &ret_code, sizeof(int));
            close(newsock);
#if !DEBUG
            exit(0);
        }

        close(newsock);
#endif
    }

}
</PRE>
</BODY></HTML>

