/* Copyright (c) 1996 by UniBwH Inst. f. Informatik.  All Rights Reserved */

/***
   NAME
     psfacct
   PURPOSE
     1. Writes an account entry for printed files
     2. copies stdin to stdout
   NOTES
     
   HISTORY
     stef - Jan 9, 1996: Created.
***/

#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <math.h>
#include <time.h>
#include "psf.h"

char line[LONG_STR + 1];         /* input line */
char *account_file_name = NULL;  /* accounting file name  */
char *host = NULL;		 /* accounting host       */
char user[10] = "\0";	
char *date = NULL;
char *printer = NULL;
char *job_name = NULL;
char *seq_number = NULL;
char *z_options = NULL;
char *format = NULL;
char *pgmname;
char *first_line;

int status, log_exit;

extern void write_accounting (long);
extern void time_stamp (void);
extern void get_time (char *);

#define CTLD_EOJ	0x04	/* ctl/d is end of postscript job.         */

int
main (int argc, char **argv)
{
  long pages_printed; 
  float result;
  char z[2];
  int c, i, page_counter_found = 0;
  int number_of_copies = 1;
  int line_counter = 0;
  int lines_on_page = 60;         /* default page length   */
  int z_opt_faktor = 1;
  char buf[BUFSIZ];
  extern char *optarg;
  extern int optind, opterr, optopt;
  FILE *input_fp, *output_fp;
  

  pages_printed = 0L;

  while ((c = getopt (argc, argv, 
		      "c:w:l:n:N:q:S:F:x:y:Y:v:P:h:J:C:D:L:Z:s:u:")) != -1)
    {
      switch (c)
	{
          case 'n':
	    (void) strcpy (user, optarg);
	    break;
          case 'h':
	    host = optarg;
	    break;
          case 'N':
	    job_name = optarg;
	    break;
          case 'P':
	    printer = optarg;
	    break;
          case 'l':
	    lines_on_page = (int) atoi (optarg);
	    break;
 	  case 'F':
	    format = optarg;
	    break;
 	  case 'Z':
	    {
	      z_options = (char *) malloc (strlen (optarg) * sizeof (char) + 1);
	      if (z_options == (char *) NULL) {
	          fprintf (stderr, "psfacct: malloc for z_option string failed.\n");
		  exit (ERROR);
	      } else {
	          (void) strcpy (z_options, optarg);
	      }
	      z[0] = z_options[0];
	      z[1] = '\0';
	      i = (int) atoi (z);
	      switch (i)
		{
	          case 2:
	          case 4:
	          case 6:
	          case 8:
		    z_opt_faktor = i;
		    break;
	        default:
		    z_opt_faktor = 1;
		} 
	    }
	    break;
          case 'q':
	    seq_number = optarg;
	    break;
          case 'D':
	    date = optarg;
	    break;
          default:
	    ;
	}
      if (optind == argc - 1)
	  account_file_name = argv[optind];
    }
 
  output_fp = stdout;
  input_fp  = stdin; 
 
  if (fgets (line, LONG_STR, input_fp) == NULL)
      exit (JOB_IS_DONE);

  first_line = line;
  while (*first_line == CTLD_EOJ)	/* skip leading ctl/d */
    first_line++;

  pages_printed++;

  if ((strncmp (first_line, "%!", 2) == 0 || strncmp (first_line, "%%!", 3) == 0)) {
   /* it's PostScript */
       fputs (line, output_fp); /* just put stdin -> stdout */      
       while (fgets (line, LONG_STR, input_fp) != NULL) { 
	   /* Checking if there is a copies operator */
	   if (strstr (line, "/#copies") != NULL) {
	       sscanf (line, "%s %d", buf, &number_of_copies);
	   }
	   /* This is for dvips */
	   if (strstr (line, "@copies")) {
	       sscanf (line, "%d", &number_of_copies);
	   }

	   if (page_counter_found) {
	       if (strstr (line, "%%Page:") != NULL) {
		   page_counter_found++;
	       }
	   } else {
	       if (strstr (line, "%%Pages:") != NULL) {
		   {
		       char *tmp_buf;
		       int l;
		       tmp_buf = (char *) malloc (256 * sizeof (char));
		       if (tmp_buf == NULL) {
			   fprintf (stderr, "psfacct: Malloc of tmp_buf failed!\n");
			   exit (1);
		       }
		       (void) strcpy (tmp_buf, line);
		       for (l = 0; l < strlen (line); l++) {
			   if (*tmp_buf == ':') {
			       break;
			   }
			   tmp_buf++;
		       }
		       tmp_buf++;
		       (void) sscanf (tmp_buf, "%ld", &pages_printed);
		       if (strstr (tmp_buf, "(atend)") != 0) {
			   page_counter_found = 1;
		       }
		   }
	       }
	   }
	   fputs (line, output_fp); /* just put stdin -> stdout */      
       }    
  } else { /* regular ASCII text */
      if (z_opt_faktor > 1) {  /* not interesting when printing text files, reset */
	  z_opt_faktor = 1;
      }
      fputs (line, output_fp); /* just put stdin -> stdout */      
      while (fgets (line, LONG_STR, input_fp) != NULL) { 
	  fputs (line, output_fp); /* just put stdin -> stdout */
	  line_counter++;
	  if (line_counter > lines_on_page) {
	      line_counter = 0;
	      pages_printed++;
	  }
      }
      if (line_counter > 0)
	  pages_printed++;
  }
  
  pages_printed = pages_printed * number_of_copies;
  result = (float) pages_printed / (float) z_opt_faktor;
  pages_printed = (long) ceil (result);
  
  write_accounting (pages_printed);
  
  if (status != 0) {
      exit (ERROR);
  }
  return (0);
}


/*
 * This routines come from the psf-3.6 suite, not from me 
 */

/****************************************************************************
 *  write_accounting ()						            *
 *  send number of printed pages to accounting file. record non-zero exit   *
 ****************************************************************************/

void
write_accounting (long pages_printed)
{
  FILE *accounting;

  if (account_file_name == NULL)
      return;

  if ((accounting = fopen (account_file_name, "a")) != NULL)
      {
	if (date == NULL) {
	    date = (char *) malloc (24 * sizeof (char));
	    if (date == NULL) {
	        fprintf (stderr, "psfacct: No memory available for date!\n");
		exit (DISCARD_JOB);
	    }
	    get_time (date);
	}
	status = fprintf (accounting, "%s\t%s\t%s\t%ld %s %s\t%s %s\n",
			  user, host, printer, pages_printed, format, job_name, 
			  seq_number, date);
	if (status > 0) { /* successfully ended */
	    status = 0;
	}
	fclose (accounting);
      }
  else
    {
      errno = 0;
      time_stamp ();
      perror (account_file_name);
      log_exit = errno;
    }
}

/****************************************************************************
 *	get_time()							    *
 *	gets the current time 				                    *
 ****************************************************************************/
void
get_time (char *t)			/* get current time of */
{
    time_t n_time;
    char *x_time, *cc;

    n_time = time ((time_t *) 0);	/* get time */
    x_time = ctime (&n_time);	/* convert ascii */
    if ((cc = strchr (x_time, '\n')) != NULL)
	*cc = '\0';
    strcpy (t, x_time);
}

/****************************************************************************
 *	time_stamp ()							    *
 *	record date stamp into error log file				    *
 ****************************************************************************/

void
time_stamp (void)
{
  int e = errno;
  time_t clock = time ((time_t *) 0);
  (void) fprintf (stderr, "%s: %.24s ", pgmname, ctime (&clock));
  errno = e;
}
