/****************************************************************************
 *			p s f c b a n n e r . c			            *
 *									    *
 *	Print a banner page on a color postscript printer		    *
 *									    *
 * The generated print stream is NOT minimally conforming as per Red Book.  *
 *									    *
 *	Tony Field.       tony@ajfcal.cuc.ab.ca				    *
 ****************************************************************************
*/

/*
 * $Id: psfcbanner.c,v 3.4 1992/02/24 06:48:59 ajf Exp ajf $
 * Modified for use with PLP by Stefano Ianigro (stef@unibw-hamburg.de)
 * Sat Mar 18, 21:22, 1995
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "ctp.h"
#include "psf.h"
#include "patchlevel.h"

/*
 * modify the following font selections if language changes are needed in
 * psfprint.def. Also enable the language translation code at line 257  (
 * if (need_xlate)  )
 */

#define FONT1 "/Bookman-Demi findfont %d scalefont setfont\n" 
#define FONT2 "/Helvetica-Bold set_font /Helvetica-Bold findfont %d scalefont setfont\n"
#define FONT3 "/Courier set_font /Courier findfont %d scalefont setfont\n"
#define FONT4 "/Helvetica-Bold set_font /Helvetica-Bold findfont %d scalefont setfont\n"
#define FONT5 "/Times-Bold findfont 40 scalefont setfont\n"

/* setup paper dimensions */

typedef struct			/* measurement in point  */
  {
    char paper_name[60];	/* name of paper size (for command
				 * line)       
				 */
    char paper_tray[200];	/* postscript operator to
				 * select this tray    
				 */
    int width;			/* portrait point width of paper   */
    int height;			/* portrait point height of paper  */
    int lx, ly, ux, uy;
  }
measure;

measure *p;			/* working set of parameters  (points to
				 * something below)    
				 */

/* standard paper sizes in portrait layout.                        */

measure page_types[NPAGE] =
{
  {"letter",
   "statusdict begin lettertray end",
   612, 792, 18, 15, 593, 777
  },
  {"legal",
   "statusdict begin legaltray end",
   612, 1008, 18, 15, 593, 993
  },
  {"a4",
   "statusdict begin a4tray end",
   595, 842, 18, 15, 578, 827
  },
  {"b5",
   "statusdict begin b5tray end",
   516, 729, 18, 15, 497, 712
  },
  {"",				/* end of paper list */
   "",
   0, 0, 0, 0, 0, 0
  }
};

char slots[NSLOTS][600] =
{
  "statusdict begin 1 setpapertray end",
  "statusdict begin 2 setpapertray end",
  "", "", "", ""
};

int nslots = 5;

int X_INDENT = 30;		/* points                                        */
int Y_INDENT = 30;
int BIG_POINT = 60;		/* changes to point size automatically scale     */
int MED_POINT = 35;		/* the printout.                                 */
int SMALL_POINT = 15;
int UX, UY, DX, DY, MAX_X, MAX_Y;
int k, l, is_qms2425;
/*
 * The following arguments are received in the command line argv[9] and
 * argv[10] may be page size and paper tray specifications. these could be:
 * 
 * -b n = select paper bin n -g size = selelct paper tray (letter, b5, etc)
 */
#ifdef USE_WITH_PLP
char Division[200];	/* division name   */
char Printer[64];	/* printer name    */
char Title[64];  	/* jobname         */
char Options[64];	/* Z-options       */
char Requestid[10]; 	/* Sequence number */
char Date[30];   	/* Date            */
char Name[64];  	/* real username   */
char Userid[9]; 	/* login name      */
char Machineid[64];	/* host            */
char User[9];   	/* login name      */
char Class[2];          /* Class           */
char buf[200];
#else /* NOT to use with PLP */
#define	Userid		argv[1]
#define	Name		argv[2]
#define	Requestid	argv[3]
#define	Printer		argv[4]
#define	Options		argv[5]
#define	Date		argv[6]
#define	Machineid	argv[7]
#define	Title		argv[8]
#endif

char *defref = NULL;

FILE *outfp;
unsigned char xlate[256];	/* translation vector    */
int need_xlate = 1;		/* default xlate      */
FILE *pdef;
void trim ();

main (argc, argv)
     int argc;
     char *argv[];
{
  int x, y, c, i, set_paper_tray, set_paper_bin, number, j;
  char line[200];
  time_t t;               /* date string */
  char * time_buf;
  char z;

  if (argc < 8) {
    usage ();
  }

  for (i = 0; i < 256; i++)
    xlate[i] = i;

  outfp = stdout;
  pdef = NULL;

/* get user specified def file with -u */

  for (i = 0; i < argc; i++)
    {
      if (strncmp (argv[i], "-u", 2) == 0)
	{
	  if (strlen (argv[i]) > 2)
	    defref = argv[i] + 2;
	  else
	    defref = argv[i + 1];
	  break;
	}
    }
  if (i >= argc)
    defref = PDEF;

  if ((pdef = fopen (defref, "r")) != NULL)
    {
      char line_type[200];

      while (fgets (line, 200, pdef))
	{
	  if (strncmp (line, "*printer", 8) == 0)
	    break;
	}
      nslots = 0;
      fgets (line, 200, pdef);	/* skip printer name     */
      if ((strstr (line, "QMS")) && (strstr (line, "2425"))) {
	  is_qms2425 = 1;
      }
      while (fgets (line, 200, pdef))
	{
	  trim (line);
	  sscanf (line, "%s%d", line_type, &number);

	  if (strcmp (line_type, "*paper") == 0)
	    {
	      for (i = j = 0; i < number; i++)
		{
		  if (i >= NPAGE - 1)
		    {
		      fgets (line, 200, pdef);
		      fgets (line, 200, pdef);
		      fgets (line, 200, pdef);
		      continue;
		    }
		  if (fgets (line, 200, pdef) == NULL)
		    bad_file ();
		  line[59] = 0;
		  trim (line);
		  strcpy (page_types[i].paper_name, line);
		  if (fgets (line, 99, pdef) == NULL)
		    bad_file ();
		  line[99] = 0;
		  trim (line);
		  strcpy (page_types[i].paper_tray, line);
		  if (fgets (line, 100, pdef) == NULL)
		    bad_file ();
		  trim (line);
		  sscanf (line, "%d%d%d%d%d%d",
			  &page_types[i].width, &page_types[i].height,
			  &page_types[i].lx, &page_types[i].ly,
			  &page_types[i].ux, &page_types[i].uy);
		  j = i;
		}
	      page_types[j + 1].paper_name[0] = '\0';
	    }
	  else if (strcmp (line_type, "*slots") == 0)
	    {
	      for (j = 0; j < NSLOTS; j++)
		slots[j][0] = 0;
	      for (i = j = 0; i < number; i++)
		{
		  if (is_qms2425) {
		      k = 3;
		      for (l = 0; l < k; l++) {
			  if (fgets (line, 200, pdef) == NULL)
			      bad_file ();
			  if (i >= NSLOTS)
			      continue;
			  /*	  line[99] = 0; */
			  trim (line);
			  if (l == 0) {
			      strcpy (slots[i], line);
			      j = i;
			  } else {
			      strcat (slots[i], "\n");
			      strcat (slots[i], line);
			  }
		      }
		  } else {  
		      if (fgets (line, 200, pdef) == NULL)
			  bad_file ();
		      if (i >= NSLOTS)
			  continue;
		      /* line[99] = 0; */
		      trim (line);
		      strcpy (slots[i], line);
		      j = i;
		  }
		}
	      nslots = j + 1;
	    }
	  else if (strcmp (line_type, "*eof") == 0)
	    break;
	}
    }
  MAX_X = page_types[0].width;
  MAX_Y = page_types[0].height;
  set_paper_bin = 0;
  set_paper_tray = -1;

#ifdef USE_WITH_PLP
  for (i = 0; i < argc; i++)
#else
  for (i = 9; i < argc; i++)
#endif
    {
      if (argv[i][0] != '-')
	continue;
      c = argv[i][1];
      switch (c)
	{
	case 'b':
	  if ((set_paper_bin = atoi (argv[i + 1])) > nslots)
	    {
	      fprintf (stderr, "paper bin greater than %d\n", nslots);
	      exit (DISCARD_JOB);
	    }
	  i++;
	  break;

	case 'g':
	  for (set_paper_tray = 0; page_types[set_paper_tray].paper_name[0]; set_paper_tray++)
	    {
	      if (compare (argv[i + 1], page_types[set_paper_tray].paper_name) == 0)
		{
		  MAX_X = page_types[set_paper_tray].width;
		  MAX_Y = page_types[set_paper_tray].height;
		  break;
		}
	    }
	  if (page_types[set_paper_tray].paper_name[0] == '\0')
	    {
	      fprintf (stderr, "Invalid paper size\n");
	      exit (DISCARD_JOB);
	    }
	  i++;
	  break;

	case 'u':		/* already processed */
	  break;
#ifdef USE_WITH_PLP
	case 'B':               /* Division name */
	    (void) sprintf (Division, "%s", argv[i]+2);
	  break;
	case 'J':               /* Job name */
	    (void) sprintf (Title, "%s", argv[i]+2);
	  break;
	case 'S':               /* Printer name */
	    (void) sprintf (Printer, "%s", argv[i]+2);
	  break;
	case 'j':
	case 'q':               /* Sequence number */
	    (void) sprintf (Requestid, "%s", argv[i]+2);
	  break;
	case 'C':               /* Class */
	    (void) sprintf (Class, "%s", argv[i]+2);
	  break;
	case 't':
	case 'D':               /* Date when job was spooled */
	    (void) sprintf (Date, "%s", argv[i]+2);
	  break;
	case 'n':               /* userid */
	    (void) sprintf (Userid, "%s",  argv[i]+2);
	    (void) sprintf (User, "%s",  argv[i]+2);
	  break;
	case 'h':               /* hostname */
	    (void) sprintf (Machineid, "%s", argv[i]+2);
	  break;
	case 'Z':               /* Z-options */
	    (void) sprintf (Options, "%s", argv[i]+2);
	  break;
	case 'L':               /* Real user name */
	    (void) sprintf (Name, "%s", argv[i]+2);
	  break;
#endif
	default:;
	}
    }

  UX = X_INDENT;
  UY = (MAX_Y - Y_INDENT);
  DX = (MAX_X - 2 * X_INDENT -25);
  DY = (BIG_POINT * 3);

/* center text used for userid and job title print  */

  send ("\n");
  send ("%!\n");
  send ("statusdict /setduplexmode known\n");
  send ("{ statusdict begin false setduplexmode end } if\n");
  send ("statusdict /settumble known\n");
  send (" { statusdict begin false settumble end } if\n");
  send ("/ISOfonts 64 dict def\n"); /* Prologue for ISO fonts */
  send ("/ISOEncoding 256 array dup 0 StandardEncoding putinterval\n");
  send ("/#copies 1 def\n");
  send ("dup 8#244 /currency put\n");
  send ("dup 8#250 /dieresis put\n");
  send ("dup 8#252 /ordfeminine put\n");
  send ("dup 8#260 /ring put\n");
  send ("dup 8#272 /ordmasculine put\n");
  send ("dup 8#300 /Agrave put\n");
  send ("dup 8#301 /Aacute put\n");
  send ("dup 8#302 /Acircumflex put\n");
  send ("dup 8#303 /Atilde put\n");
  send ("dup 8#304 /Adieresis put\n");
  send ("dup 8#305 /Aring put\n");
  send ("dup 8#306 /AE put\n");
  send ("dup 8#307 /Ccedilla put\n");
  send ("dup 8#310 /Egrave put\n");
  send ("dup 8#311 /Eacute put\n");
  send ("dup 8#312 /Ecircumflex put\n");
  send ("dup 8#313 /Edieresis put\n");
  send ("dup 8#314 /Igrave put\n");
  send ("dup 8#315 /Iacute put\n");
  send ("dup 8#316 /Icircumflex put\n");
  send ("dup 8#317 /Idieresis put\n");
  send ("dup 8#321 /Ntilde put\n");
  send ("dup 8#322 /Ograve put\n");
  send ("dup 8#323 /Oacute put\n");
  send ("dup 8#324 /Ocircumflex put\n");
  send ("dup 8#325 /Otilde put\n");
  send ("dup 8#326 /Odieresis put\n");
  send ("dup 8#330 /Oslash put\n");
  send ("dup 8#331 /Ugrave put\n");
  send ("dup 8#332 /Uacute put\n");
  send ("dup 8#333 /Ucircumflex put\n");
  send ("dup 8#334 /Udieresis put\n");
  send ("dup 8#337 /germandbls put\n");
  send ("dup 8#340 /agrave put\n");
  send ("dup 8#341 /aacute put\n");
  send ("dup 8#342 /acircumflex put\n");
  send ("dup 8#343 /atilde put\n");
  send ("dup 8#344 /adieresis put\n");
  send ("dup 8#345 /aring put\n");
  send ("dup 8#346 /ae put\n");
  send ("dup 8#347 /ccedilla put\n");
  send ("dup 8#350 /egrave put\n");
  send ("dup 8#351 /eacute put\n");
  send ("dup 8#352 /ecircumflex put\n");
  send ("dup 8#353 /edieresis put\n");
  send ("dup 8#354 /igrave put\n");
  send ("dup 8#355 /iacute put\n");
  send ("dup 8#356 /icircumflex put\n");
  send ("dup 8#357 /idieresis put\n");
  send ("dup 8#361 /ntilde put\n");
  send ("dup 8#362 /ograve put\n");
  send ("dup 8#363 /oacute put\n");
  send ("dup 8#364 /ocircumflex put\n");
  send ("dup 8#365 /otilde put\n");
  send ("dup 8#366 /odieresis put\n");
  send ("dup 8#370 /oslash put\n");
  send ("dup 8#371 /ugrave put\n");
  send ("dup 8#372 /uacute put\n");
  send ("dup 8#373 /ucircumflex put\n");
  send ("dup 8#374 /udieresis put\n");
  send ("dup 8#377 /ydieresis put\n");
  send ("\n");
  send ("def\n");
  send ("\n");
  send ("ISOfonts /Symbol dup findfont put\n");
  send ("/ISOfont\n");
  send ("{\n");
  send (" dup ISOfonts exch known not\n");
  send (" {\n");
  send ("  ISOfonts begin\n");
  send ("  dup dup findfont\n");
  send ("  dup length 5 add dict /newdict exch def\n");
  send ("  {1 index /FID ne\n");
  send ("   {newdict 3 1 roll put}\n");
  send ("   {pop pop}\n");
  send ("   ifelse\n");
  send ("  }forall\n");
  send ("  newdict /Encoding ISOEncoding put\n");
  send ("  dup newdict definefont def\n");
  send ("  end\n");
  send (" }if\n");
  send (" ISOfonts exch get\n");
  send ("}def\n");
  send ("\n");
  send ("/set_font{ISOfont setfont}def\n");
  send ("\n");
  send ("/ctextu     % center text:   string x y dx\n");
  send ("{\n");
  send ("   2 div\n");
  send ("   /Dx exch def /Yv exch def /Xv exch def\n");
  send ("   dup stringwidth pop\n");
  send ("   2 div Dx exch sub Xv add Yv\n");
  send ("   gsave translate 0.6 setlinewidth 0 0 moveto\n");
  send ("    {\n");
  send ("       /charcode exch def\n");
  send ("       /thechar (-) dup 0 charcode put def\n");
  send ("       thechar true charpath gsave 0.8 setgray fill grestore stroke\n");
  send ("       thechar stringwidth pop 0 translate 0 0 moveto\n");
  send ("    } forall\n");
  send ("    grestore\n");
  send ("} def\n");
  send ("/ctext {     % center text:   string x y dx\n");
  send ("   2 div\n");
  send ("   /Dx exch def /Yv exch def /Xv exch def\n");
  send ("   dup stringwidth pop\n");
  send ("   2 div Dx exch sub Xv add Yv moveto\n");
  send ("   show\n");
  send ("} def\n");
  replacement_fonts ();		/* get font translations */
  send ("%%EndProlog\n");

/*
 * Uncomment the following code if languange translation is needed for the
 * following parameters
 */

  if (need_xlate)
    {
#ifdef USE_WITH_PLP
      xlate_string (Userid);
      xlate_string (Division);
      xlate_string (Name);
      xlate_string (Requestid);
      xlate_string (Printer);
      xlate_string (Options);
      xlate_string (Date);
       z = Machineid[0];
      /* the hostname begins with a capitelized letter */
      if (z > 96) {
	  z = z - 32;
	  Machineid[0] = z;
      }
      xlate_string (Machineid);
      xlate_string (Title);
#else
      xlate_string (Userid);
      xlate_string (Name);
      xlate_string (Requestid);
      xlate_string (Printer);
      xlate_string (Options);
      xlate_string (Date);
      xlate_string (Machineid);
      xlate_string (Title);
#endif
    }
  send ("%%Page: ? 1\n");
  if (set_paper_bin)
    printf ("%s\n", slots[set_paper_bin - 1]);

  if (set_paper_tray >= 0)
    printf ("%s\n", page_types[set_paper_tray].paper_tray);

  send ("/pg save def\n");

/* draw a box for the userid */

  printf ("newpath\n");
  printf ("%d %d moveto\n", UX, UY);
  printf ("%d %d rlineto\n", DX, 0);
  printf ("%d %d rlineto\n", 0, -DY);
  printf ("%d %d rlineto\n", -DX, 0);
  printf ("closepath\n");
  printf ("4 setlinewidth\n");
  printf ("stroke\n");

/* center the userid and the job title */

  y = UY - DY / 2 - BIG_POINT / 2;
  printf (FONT1, BIG_POINT);
  printf ("gsave\n/col1 {0.000 0.000 1.000 setrgbcolor} bind def\n");
  printf ("(%s) col1 %d %d %d ctextu grestore\n", Userid, X_INDENT, y, DX);

  y = UY - (DY + MED_POINT * 3);
  printf (FONT2, MED_POINT);
  printf ("(%s) %d %d %d ctext\n", Title, X_INDENT, y, DX);

/* print other banner page parameters */

/* sequence number */
#ifdef USE_WITH_PLP
  printf ("%s", FONT5);
  printf ("520 20 moveto\n");
  printf ("(%s)show\n", Requestid);
  printf ("520 800 moveto\n");
  printf ("(%s)show\n", Requestid);
  printf ("27 20 moveto\n");
  printf ("(%s)show\n", Requestid);
  printf ("27 800 moveto\n");
  printf ("(%s)show\n", Requestid);
#endif

  x = X_INDENT;
  y -= (MED_POINT * 2);
  printf ("%d %d moveto\n", x, y);
  sendnormal ("User:       ");
  sendbold (User);

#ifdef USE_WITH_PLP
  y -= SMALL_POINT + (SMALL_POINT / 2);
  printf ("%d %d moveto\n", x, y);
  sendnormal ("Name:       ");
  sendbold (Name);
#endif

  y -= SMALL_POINT + (SMALL_POINT / 2);
  printf ("%d %d moveto\n", x, y);
  sendnormal ("Request ID: ");
  sendbold (Requestid);

  y -= SMALL_POINT + (SMALL_POINT / 2);
  printf ("%d %d moveto\n", x, y);
  sendnormal ("Printer ID: ");
  sendbold (Printer);

#ifdef USE_WITH_PLP
  y -= SMALL_POINT + (SMALL_POINT / 2);
  printf ("%d %d moveto\n", x, y);
  sendnormal ("Job:        ");
  sendbold (Title);
#endif

  y -= SMALL_POINT + (SMALL_POINT / 2);
  printf ("%d %d moveto\n", x, y);
  sendnormal ("Options:    ");
  sendbold (Options);

#ifdef USE_WITH_PLP
  y -= SMALL_POINT + (SMALL_POINT / 2);
  printf ("%d %d moveto\n", x, y);
  sendnormal ("Class:      ");
  sendbold (Class);

  y -= SMALL_POINT + (SMALL_POINT / 2);
  printf ("%d %d moveto\n", x, y);
  sendnormal ("Spooled at: ");
  sendbold (Date);

  y -= SMALL_POINT + (SMALL_POINT / 2);
  t = time ((time_t *) NULL);     /* initialize printing date */
  time_buf = (char *) malloc (strlen(ctime (&t) + 1) * sizeof (char));
  if (time_buf == NULL ) {
      fprintf (stderr, "psfcbanner: Error allocating memory for time_buf, aborting\n");
      exit (1);
  }
  printf ("%d %d moveto\n", x, y);
  sendnormal ("Printed at: ");
  time_buf = ctime (&t);
  time_buf[24] = '\0';   /* strip the \n character */
  sendbold (time_buf);
#else
  y -= SMALL_POINT + (SMALL_POINT / 2);
  printf ("%d %d moveto\n", x, y);
  sendnormal ("Date:       ");
  sendbold (Date);
#endif /* USE_WITH_PLP */

  y -= SMALL_POINT + (SMALL_POINT / 2);
  printf ("%d %d moveto\n", x, y);
  sendnormal ("Machine:    ");
  sendbold (Machineid);

#ifdef USE_WITH_PLP
  y -= SMALL_POINT + (SMALL_POINT / 2);
  printf ("%d %d moveto\n", x, y);
  sendnormal ("Division:   ");
  sendbold_i (Division);

/* The PLP logo... */
  printf ("gsave\n");
  printf ("/$F2psDict 200 dict def\n");
  printf ("$F2psDict begin\n");
  printf ("$F2psDict /mtrx matrix put\n");
  printf ("/col-1 {0 setgray} bind def\n");
  printf ("/col0 {0.000 0.000 0.000 srgb} bind def\n");
  printf ("/col1 {0.000 0.000 1.000 srgb} bind def\n");
  printf ("/col2 {0.000 1.000 0.000 srgb} bind def\n");
  printf ("/col3 {0.000 1.000 1.000 srgb} bind def\n");
  printf ("/col4 {1.000 0.000 0.000 srgb} bind def\n");
  printf ("/col5 {1.000 0.000 1.000 srgb} bind def\n");
  printf ("/col6 {1.000 1.000 0.000 srgb} bind def\n");
  printf ("/col7 {1.000 1.000 1.000 srgb} bind def\n");
  printf ("/col8 {0.000 0.000 0.560 srgb} bind def\n");
  printf ("/col9 {0.000 0.000 0.690 srgb} bind def\n");
  printf ("/col10 {0.000 0.000 0.820 srgb} bind def\n");
  printf ("/col11 {0.530 0.810 1.000 srgb} bind def\n");
  printf ("/col12 {0.000 0.560 0.000 srgb} bind def\n");
  printf ("/col13 {0.000 0.690 0.000 srgb} bind def\n");
  printf ("/col14 {0.000 0.820 0.000 srgb} bind def\n");
  printf ("/col15 {0.000 0.560 0.560 srgb} bind def\n");
  printf ("/col16 {0.000 0.690 0.690 srgb} bind def\n");
  printf ("/col17 {0.000 0.820 0.820 srgb} bind def\n");
  printf ("/col18 {0.560 0.000 0.000 srgb} bind def\n");
  printf ("/col19 {0.690 0.000 0.000 srgb} bind def\n");
  printf ("/col20 {0.820 0.000 0.000 srgb} bind def\n");
  printf ("/col21 {0.560 0.000 0.560 srgb} bind def\n");
  printf ("/col22 {0.690 0.000 0.690 srgb} bind def\n");
  printf ("/col23 {0.820 0.000 0.820 srgb} bind def\n");
  printf ("/col24 {0.500 0.190 0.000 srgb} bind def\n");
  printf ("/col25 {0.630 0.250 0.000 srgb} bind def\n");
  printf ("/col26 {0.750 0.380 0.000 srgb} bind def\n");
  printf ("/col27 {1.000 0.500 0.500 srgb} bind def\n");
  printf ("/col28 {1.000 0.630 0.630 srgb} bind def\n");
  printf ("/col29 {1.000 0.750 0.750 srgb} bind def\n");
  printf ("/col30 {1.000 0.880 0.880 srgb} bind def\n");
  printf ("/col31 {1.000 0.840 0.000 srgb} bind def\n");
  printf ("/col32 {1.000 0.890 0.784 srgb} bind def\n");
  printf ("/col33 {0.486 0.341 0.000 srgb} bind def\n");
  printf ("/col34 {0.745 0.392 0.000 srgb} bind def\n");
  printf ("\n");
  printf ("end\n");
  printf ("save\n");
  printf ("-5 175.0 translate 0.900 -0.900 scale\n");
  printf ("\n");
  printf ("/cp {closepath} bind def\n");
  printf ("/ef {eofill} bind def\n");
  printf ("/gr {grestore} bind def\n");
  printf ("/gs {gsave} bind def\n");
  printf ("/sa {save} bind def\n");
  printf ("/rs {restore} bind def\n");
  printf ("/l {lineto} bind def\n");
  printf ("/m {moveto} bind def\n");
  printf ("/rm {rmoveto} bind def\n");
  printf ("/n {newpath} bind def\n");
  printf ("/s {stroke} bind def\n");
  printf ("/sh {show} bind def\n");
  printf ("/slc {setlinecap} bind def\n");
  printf ("/slj {setlinejoin} bind def\n");
  printf ("/slw {setlinewidth} bind def\n");
  printf ("/srgb {setrgbcolor} bind def\n");
  printf ("/rot {rotate} bind def\n");
  printf ("/sc {scale} bind def\n");
  printf ("/sd {setdash} bind def\n");
  printf ("/ff {findfont} bind def\n");
  printf ("/sf {setfont} bind def\n");
  printf ("/scf {scalefont} bind def\n");
  printf ("/sw {stringwidth} bind def\n");
  printf ("/tr {translate} bind def\n");
  printf ("/tnt {dup dup currentrgbcolor\n");
  printf ("4 -2 roll dup 1 exch sub 3 -1 roll mul add\n");
  printf ("4 -2 roll dup 1 exch sub 3 -1 roll mul add\n");
  printf ("4 -2 roll dup 1 exch sub 3 -1 roll mul add srgb}\n");
  printf ("bind def\n");
  printf ("/shd {dup dup currentrgbcolor 4 -2 roll mul 4 -2 roll mul\n");
  printf ("4 -2 roll mul srgb} bind def\n");
  printf ("/$F2psBegin {$F2psDict begin /$F2psEnteredState save def} def\n");
  printf ("/$F2psEnd {$F2psEnteredState restore end} def\n");
  printf ("%%%%EndProlog\n");
  printf ("\n");
  printf ("$F2psBegin\n");
  printf ("10 setmiterlimit\n");
  printf ("n 0 612 m 0 0 l 792 0 l 792 612 l cp clip\n");
  printf ("0.06299 0.06299 sc\n");
  printf ("7.500 slw\n");
  printf ("%% Polyline\n");
  printf ("n 1935 1935 m 1935 1800 l 900 1800 l 900 1935 l cp gs 0.00 setgray ef gr gs col-1 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 900 1800 m 900 1575 l 675 1575 l 675 1800 l cp gs 0.00 setgray ef gr gs col-1 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 2160 1800 m 2160 1575 l 1935 1575 l 1935 1800 l cp gs 0.00 setgray ef gr gs col-1 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 765 1440 m 765 1395 l 675 1395 l 675 1440 l cp gs 0.00 setgray ef gr gs col-1 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 2070 1440 m 2070 1395 l 2160 1395 l 2160 1440 l cp gs 0.00 setgray ef gr gs col-1 s gr \n");
  printf ("15.000 slw\n");
  printf ("%% Polyline\n");
  printf ("n 2205 1170 m 2205 1350 l 630 1350 l 630 1170 l cp gs col-1 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 660 1365 m 660 1185 l gs col-1 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 2175 1350 m 2175 1170 l gs col-1 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 630 1170 m 630 1080 l gs col-1 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 2205 1170 m 2205 1080 l gs col-1 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 765 990 m 900 990 l gs col-1 s gr \n");
  printf ("7.500 slw\n");
  printf ("%% Polyline\n");
  printf ("n 900 1065 m 1935 1065 l gs col-1 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 900 630 m 900 1080 l gs col-1 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 900 1080 m 1935 1080 l gs col-1 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 900 630 m 1845 630 l gs col-1 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1935 735 m 1935 1080 l gs col-1 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1845 630 m 1935 735 l gs col-1 s gr \n");
  printf ("15.000 slw\n");
  printf ("%% Polyline\n");
  printf ("n 1935 990 m 1980 990 l gs col-1 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1950 990 m 2115 990 l 2205 1095 l gs col-1 s gr \n");
  printf ("7.500 slw\n");
  printf ("%% Polyline\n");
  printf ("n 1935 1005 m 1980 1005 l gs col-1 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1935 1020 m 1980 1020 l gs col-1 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 2010 1005 m 2115 1005 l gs col-1 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 2010 1020 m 2130 1020 l gs col-1 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1980 1035 m 2010 1035 l gs col-1 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1980 1050 m 2010 1050 l gs col-1 s gr \n");
  printf ("15.000 slw\n");
  printf ("%% Polyline\n");
  printf ("n 780 990 m 630 1095 l gs 0.85 setgray ef gr gs col0 s gr \n");
  printf ("7.500 slw\n");
  printf ("%% Polyline\n");
  printf ("n 1995 1170 m 2190 1170 l 2205 1170 l 2205 1080 l 2205 1095 l 2130 1005 l\n");
  printf ("2010 1005 l 2010 1035 l 2010 1035 l 2010 1050 l 1980 1050 l gs 0.75 setgray ef gr gs col0 s gr \n");
  printf ("15.000 slw\n");
  printf ("%% Polyline\n");
  printf ("n 2130 1350 m 2130 1395 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 705 1350 m 705 1410 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 990 1545 m 1050 1545 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1875 1545 m 1815 1545 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1830 1545 m 1800 1545 l gs col0 s gr \n");
  printf ("7.500 slw\n");
  printf ("%% Polyline\n");
  printf ("n 975 1575 m 975 1635 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1875 1575 m 1875 1635 l gs col0 s gr \n");
  printf ("15.000 slw\n");
  printf ("%% Polyline\n");
  printf ("n 1095 1530 m 1095 1590 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1260 1530 m 1260 1590 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1290 1530 m 1290 1590 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1560 1530 m 1560 1590 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1590 1530 m 1590 1590 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1680 1530 m 1680 1590 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1755 1530 m 1755 1590 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1365 1530 m 1365 1590 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1395 1530 m 1395 1590 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1455 1530 m 1455 1590 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1485 1530 m 1485 1590 l gs col0 s gr \n");
  printf ("7.500 slw\n");
  printf ("%% Polyline\n");
  printf ("n 1995 1170 m 630 1170 l 630 1095 l 780 990 l 900 990 l 900 1080 l\n");
  printf ("1935 1080 l 1935 1020 l 1980 1020 l gs 0.75 setgray ef gr gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 2160 1335 m 675 1335 l 675 1185 l 2160 1185 l cp gs col32 1.00 shd ef gr gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 2205 1395 m 2205 1755 l 2160 1755 l 2160 1395 l cp gs 0.00 setgray ef gr gs col-1 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 675 1395 m 675 1755 l 630 1755 l 630 1395 l cp gs 0.00 setgray ef gr gs col-1 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1935 1575 m 2160 1575 l 2160 1440 l 2070 1440 l 2070 1395 l 2130 1395 l\n");
  printf ("2130 1395 l 2115 1395 l 2130 1395 l 2130 1350 l 705 1350 l\n");
  printf ("705 1395 l 765 1395 l 765 1440 l 675 1440 l 675 1575 l\n");
  printf ("900 1575 l gs col34 0.50 shd ef gr gs col0 s gr \n");
  printf ("15.000 slw\n");
  printf ("%% Polyline\n");
  printf ("n 2085 1215 m 2115 1215 l gs col2 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 2085 1290 m 2115 1290 l gs col2 s gr \n");
  printf ("7.500 slw\n");
  printf ("%% Polyline\n");
  printf ("n 1755 630 m 1755 810 l 1935 810 l gs col32 1.00 shd ef gr gs col-1 s gr \n");
  printf ("%% Polyline\n");
  printf ("[15 50.0] 50.0 sd\n");
  printf ("n 1766 641 m 1928 803 l gs col0 s gr  [] 0 sd\n");
  printf ("%% Polyline\n");
  printf ("n 870 1635 m 960 1635 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1890 1635 m 1980 1635 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 900 1575 m 900 1800 l 1935 1800 l 1935 1575 l gs col33 1.00 shd ef gr gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 975 1650 m 975 1710 l gs col0 s gr \n");
  printf ("15.000 slw\n");
  printf ("%% Polyline\n");
  printf ("n 1020 1665 m 1080 1665 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1110 1665 m 1155 1665 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1185 1665 m 1245 1665 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1305 1665 m 1350 1665 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1500 1665 m 1545 1665 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1425 1635 m 1425 1680 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1605 1665 m 1665 1665 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1695 1665 m 1740 1665 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1770 1665 m 1830 1665 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1605 1605 m 1665 1605 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1500 1605 m 1545 1605 l gs col0 s gr \n");
  printf ("7.500 slw\n");
  printf ("%% Polyline\n");
  printf ("n 1350 1635 m 1410 1635 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1245 1635 m 1305 1635 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1155 1635 m 1185 1635 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1080 1635 m 1110 1635 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 975 1635 m 1035 1635 l gs col0 s gr \n");
  printf ("15.000 slw\n");
  printf ("%% Polyline\n");
  printf ("n 1020 1605 m 1080 1605 l gs col0 s gr \n");
  printf ("7.500 slw\n");
  printf ("%% Polyline\n");
  printf ("n 975 1605 m 975 1515 l 1875 1515 l 1875 1605 l gs 0.00 setgray ef gr gs col0 s gr \n");
  printf ("15.000 slw\n");
  printf ("%% Polyline\n");
  printf ("n 1305 1605 m 1350 1605 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1185 1605 m 1245 1605 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1110 1605 m 1155 1605 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1695 1605 m 1740 1605 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1770 1605 m 1830 1605 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1425 1575 m 1425 1620 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1170 1530 m 1170 1590 l gs col0 s gr \n");
  printf ("7.500 slw\n");
  printf ("%% Polyline\n");
  printf ("n 1440 1635 m 1500 1635 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1545 1635 m 1605 1635 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1665 1635 m 1695 1635 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1740 1635 m 1770 1635 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1815 1635 m 1875 1635 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1876 1748 m 1876 1808 l gs col0 s gr \n");
  printf ("15.000 slw\n");
  printf ("%% Polyline\n");
  printf ("n 975 1620 m 975 1530 l 1875 1530 l 1875 1620 l gs col0 s gr \n");
  printf ("7.500 slw\n");
  printf ("%% Polyline\n");
  printf ("n 1875 1650 m 1875 1710 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 975 1605 m 975 1665 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 1875 1605 m 1875 1665 l gs col0 s gr \n");
  printf ("%% Polyline\n");
  printf ("n 975 1755 m 975 1815 l gs col0 s gr \n");
  printf ("/Times-Roman ff 360.00 scf sf\n");
  printf ("1050 975 m\n");
  printf ("gs 1 -1 sc (PLP) col4 sh gr\n");
  printf ("$F2psEnd\n");
  printf ("gs\n");
  printf ("155 90 moveto\n");
  printf ("1 -1 scale\n");
  printf ("/Courier set_font /Courier findfont 12 scalefont setfont\n");
  printf ("(Output generated by PLP software and friends)show\n");
#endif /* USE_WITH_PLP */
  
  send ("showpage pg \n");
  send ("%%Trailer\n");
  send ("%%EOF\n");
  exit (JOB_IS_DONE);
}


/****************************************************************************
 *	send font selection information and text.			    *
 ****************************************************************************
 */

send (s)
    char *s;
{
    while (*s)
	fputc (*s++, outfp);
}

sendnormal (s)
    char *s;
{
    printf (FONT3, SMALL_POINT);
    send ("(");
    send (s);
    send (")show\n");
}

sendbold (s)
    char *s;
{
    printf (FONT4, SMALL_POINT);
    send ("(");
    send (s);
    send (")show\n");
}

sendbold_i (s)
    char *s;
{
    printf (FONT4, SMALL_POINT);
    send ("(");
    printf ("%s\n", s);
    send (")show\n");
}

/****************************************************************************
 *	trim ()								    *
 *	trucate garbage							    *
 ****************************************************************************
 */

void
trim (s)			/* trim trailing blanks  and \n */
    char *s;
{
    int many;
    
    for (many = strlen (s) - 1; many >= 0; many--)
	{
	    if (isgraph (s[many]))
		break;
	    else
		s[many] = '\0';
	}
}

/****************************************************************************
 *	bad_file()							    *
 ****************************************************************************
 */

bad_file ()
{
  fprintf (stderr, "Bad %s file\n", defref);
  exit (DISCARD_JOB);
}

/****************************************************************************
 *	compare ()							    *
 *	partial compare of two strings.					    *
 ****************************************************************************
*/

compare (a, b)
    char *a, *b;
{
    int aa, bb;

    while (*a && *b)
	{
	    aa = *a++;
	    bb = *b++;
	    if (UCCHAR (aa) != UCCHAR (bb))
		return (aa - bb);
	}
    return (0);
}


/****************************************************************************
 *	replacement_fonts()					            *
 *	read the text translation strings only.				    *
 ****************************************************************************
*/

replacement_fonts ()
{
    int found, xold, xnew;
    char line[201];
    char *x;
    
    if (pdef == NULL)
	return;
    found = 0;
    while (fgets (line, 200, pdef))	/* skip printer name     */
	{
	    if (strncmp (line, "%%PsfStart", 10) == 0)
		{
		    found = 1;
		    break;
		}
	}
    if (found)
	{
	    while (fgets (line, 200, pdef))	/* read fonts, translate */
		{
		    if (strncmp (line, "%%PsfEnd", 8) == 0)
			break;
		    if (strncmp (line, "%%translate", 11) == 0)
			{
			    x = strtok (line + 11, " ");
			    do
				{
				    xold = otoi (x) % 256;
				    xnew = otoi (x + 4) % 256;
				    xlate[xold] = xnew;
				}
			    while (x = strtok (NULL, " "));
			    need_xlate = 1;
			}
		}
	    fclose (pdef);
	    pdef = NULL;
	}
    else
	{
	    fclose (pdef);
	    pdef = NULL;
	}
    return;
}

/****************************************************************************
 *	octal to integer						    *
 ****************************************************************************
*/

int
otoi (s)
    char *s;
{
    int v, p, quit;
    
    v = 0;
    while (*s == ' ')
	s++;
    quit = 0;
    while (*s)
	{
	    switch (*s)
		{
		case '0':
		    p = 0;
		    break;
		case '1':
		    p = 1;
		    break;
		case '2':
		    p = 2;
		    break;
		case '3':
		    p = 3;
		    break;
		case '4':
		    p = 4;
		    break;
		case '5':
		    p = 5;
		    break;
		case '6':
		    p = 6;
		    break;
		case '7':
		    p = 7;
		    break;
		default:
		    quit = 1;
		}
	    if (quit)
		break;
	    v = (v << 3) + p;
	}
    return (v);
}


/****************************************************************************
 *	xlate_string ()							    *
 *	translate the string according to the language translation vector   *
 ****************************************************************************
*/
xlate_string (s)
    unsigned char *s;
{
    int sold;
    
    while (*s)
	{
	    sold = *s;
	    *s = xlate[sold];
	    s++;
    }
}


/****************************************************************************
 *	usage()								    *
 ****************************************************************************
 */
usage ()
{
    fprintf (stderr, "Usage:  psfcbanner Userid Name Requestid Printer Options Date\n");
    fprintf (stderr, "                   Machineid Title [-b n] [-g size] [-u f.def]\n");
    fprintf (stderr, "        where:   Userid, Name, etc are passed by lpsched\n");
    fprintf (stderr, "                 -b n    = select paper bin n\n");
    fprintf (stderr, "                 -g size = select paper size (letter, b5, etc)\n");
    fprintf (stderr, "                -u f.def = use this printer definition file\n");
    exit (DISCARD_JOB);
}
