/***************************************************************************
 * U. Minnesota LPD Software * Copyright 1987, 1988, Patrick Powell
 ***************************************************************************
 * MODULE: termclear.c
 ***************************************************************************
 * 1. Do the initialization if neccessary
 * 2. clear the screen using the termcap information
 ***************************************************************************/

#include "lp.h"
#include <stdio.h>

#ifdef TERMCAP
#ifdef HAVE_TERMCAP_H
#include <termcap.h>
#endif
#endif

#if defined(TERMCAP) || defined(TERMINFO)
static int tinit_done;
static char *te;
static char *cl;

#ifdef TGETENT_VOIDP_ARG			/* argh. */
typedef void tgetent_buf_t;
#else
typedef char tgetent_buf_t;
#endif
#ifdef TPUTS_RETURNS_VOID
typedef void tputs_t;
#else
typedef int tputs_t;
#endif

#ifndef IS_LINUX /* LinuX has that in termcap.h */
extern int tgetent(tgetent_buf_t *, const char *);
extern tputs_t tputs(const char *, int, int (*)(int));
#endif 

extern char *tgetstr(const char *, char **);


/*
 * Michael Haardt: stop users from overwriting buffer.
 * this won't stop them, but it'll make it harder.
 */
static char bp[2048];
static char xp[2048];
static char *area = xp;

static int 
pputs (int c) 
{
  return (putchar(c));
}

void 
Term_clear (void) {
    char *sp, *ti;

    if (tinit_done == 0) 
    {
	tinit_done = 1;
	if (isatty (0) && (sp = getenv ("TERM")) != (char*) 0) 
	{
	    if (tgetent ((tgetent_buf_t *) bp, sp) > 0) {
		if ((ti= tgetstr ("ti", &area)) != (char*)0)
		    (void) tputs (ti, 0, pputs);
		te = tgetstr ("te", &area);
		cl = tgetstr ("cl", &area);
	    }
	}
    }
    if (cl) 
      (void) tputs (cl, 0, pputs);
    fflush(stdout);
}

void 
Term_finish (void)
{
    if (te) 
      (void) tputs (te, 0, pputs);
    fflush(stdout);
}

#else

void 
Term_clear (void) 
{
  ;
}

void 
Term_finish (void) 
{
  ;
}

#endif
