/***************************************************************************
 * U. Minnesota LPD Software * Copyright 1987, 1988, Patrick Powell
 * version 3.3.0 Justin Mason July 1994
 ***************************************************************************
 * MODULE: get_printer.c
 * gets the default printer value for a user job
 ***************************************************************************
 * Get_Printer()
 * Get the Printer Name
 * 1. check for -P option
 * 2. check for PRINTER environment variable
 * 3. use first in printcap
 *
 ***************************************************************************/

#include "lp.h"
#include "library/errormsg.h"

#ifndef DEFAULT_PRINTER
#define DEFAULT_PRINTER	"lp"
#endif

void
Get_Printer (int all) {
    int i;

    if (Printer == 0) {
	if ((Printer = getenv ("PRINTER")) != NULL) {
	    DEBUG4 ("Get_Printer: PRINTER='%s'", Printer);
	} else if
	    ((i = Get_pc_entry (DEFAULT_PRINTER,
			All_pc_vars, All_pc_len)) != 0)
	{
	    Printer = DEFAULT_PRINTER;
	    DEBUG4 ("Get_Printer: default '%s'", Printer);

	} else if (Use_first_printer) {
	    /* use the default, first in list */
	    Printer = First_printer ();
	    DEBUG4 ("Get_Printer: First_printer '%s'", Printer);

	} else {
	    Diemsg ("no printer specified");
	}
    }
	DEBUG4 ("Get_printer: using Printer %s", Printer);
    if (all) {
	i = Get_pc_entry (Printer, All_pc_vars, All_pc_len);
    } else {
	i = Get_pc_entry (Printer, Status_pc_vars, Status_pc_len);
    }
    if (i == 0) {
	Diemsg ("no printcap entry for printer %s", Printer);
    }
}
