/*************************************************************************
 * U. Minnesota LPD Software * Copyright 1987, 1988, Patrick Powell
 * version 3.3.0 Justin Mason July 1994
 * version 4.1.2 Stefano Ianigro June 1998
 *************************************************************************
 * MODULE: lp.h
 * Includes and definitions for all programs
 *************************************************************************/

#ifndef PLP_LP_H
#define PLP_LP_H

#include "portable.h"

#ifdef PLP_PROTOTYPES
/* it's tricky declaring protos for funcs with args; const may or may
 * not be used by the system include files, causing the compile to barf
 * either way. I've tried to avoid prototyping anything more than
 * the return value, but it still breaks on Linux.
 */
extern char *getenv (char *), *ctime (time_t *);
extern char *strdup (char *), *strndup (char *, unsigned int);
extern char *strchr(char *, int), *strrchr (char *, int), *strtok(char *, char *);
extern pid_t getpid (void), getppid (void), fork (void);
extern uid_t getuid (void), geteuid (void);
extern struct passwd *getpwnam (char *);
extern FILE *fdopen (int, const char *);
extern void exit (int);
extern double atof(char *);
extern long int atol (char *);
extern int isatty(), open(char *, int, mode_t), atoi(char *);
extern int fileno (FILE *), kill (pid_t, int);
extern int strlen (char *);
#endif

/*************************************************************************
 * PRINTCAP entries and associated variables.
 * Each printcap entry has a corresponding variable; for example
 * ":af=acct:"
 *   -> AF = "acct";
 *************************************************************************/
extern int AB;			/* Always have a banner flag, ignore lpr -h option */
extern char *AF;		/* accounting file */
extern int AR;			/* remote printer accounting */
extern char *BP;		/* banner printer filter */
extern int BB;                  /* no banner page should be printed between jobs */
extern int BK;			/* Berkeley compatible remote and local */
extern int BR;			/* baud rate if lp is a tty */
extern int CL;			/* close device between jobs */
extern char *CM;		/* status line comment */
extern int CO;			/* cost of printing in dollars */
extern int DB;			/* Set debug level for queue handler */
char *DS;			/* description of printer, may be needed by lpstat */
extern char *EP;		/* end printer filter */
extern int EU;			/* Edinburgh University control file format */
extern int FB;			/* use BSD ifilter+ofilter semantics */
extern int FD;			/* No forwarding, accept only from original site */
extern int FO;			/* print a form feed when device is opened */
extern int FC;			/* flags to clear if lp is a tty */
extern char *FF;		/* form feed string */
extern int FJ;			/* send control file  first to remote site */
extern int FQ;			/* form feed on quitting */
extern int FS;			/* flags to set if lp is a tty */
extern char *FX;		/* allowable formats */
extern int HL;			/* print banner after job instead of before */
extern char *LD;		/* leader string on opening */
extern char *LF;		/* log file for error messages */
extern int LA;			/* local printer accounting (by PLP) */
extern int LH;			/* use long host name */
extern char *LI;		/* file size limit */
extern char *LO;		/* lock file name */
extern char *LN;		/* group allowed to use links */
extern char *LP;		/* line printer device name */
extern int LS;                  /* flag for banner page in the lprps package */
extern int MC;			/* maximum number of copies allowed */
extern char *MI;		/* min free space to leave in spool dir */
extern int ML;			/* min legal chars for printable */
extern char *MS;		/* new stty options - as per sun */
extern unsigned long MX;	/* maximum number of blocks to copy */
extern char *OH;		/* only for the named host */
extern char *OP;		/* options for the pr program */
extern int NB;                  /* no end banner page should be printed */
extern int NU;			/* not usable as a "real" printer queue */
extern int NW;			/* Networked file system, do not make copies */
extern int PL;			/* page length */
extern int PO;			/* remote port to connect to */
extern char *PR;		/* pr program name */
extern char *PS;		/* printer status file name */
extern char *PT;		/* printer type, may be needed by lpstat */
extern int PW;			/* page width */
extern int PX;			/* page width in pixels */
extern int PY;			/* page length in pixels */
extern char *QH;		/* queue handler */
extern int RB;                  /* sending banner pages to the remote printer */
extern char *RG;		/* restrict use to group */
extern char *RM;		/* remote machine name */
extern char *RP;		/* remote printer name */
extern int RT;			/* max retries */
extern int RW;			/* open LP for reading and writing */
extern int SA;			/* indicates that this queue is the save-area */
extern int SB;			/* short banner instead of normal header */
extern int SC;			/* suppress multiple copies */
extern char *SD;		/* spool directory */
extern int SF;			/* suppress FF on each print job */
extern int SH;			/* suppress header page */
extern char *SR;		/* default format for this queue (f, l or p) */
extern char *SS;		/* name of queue that the server serves  */
extern char *ST;		/* status file name */
extern char *SV;		/* names of servers */
extern int SZ;                  /* flag for sorting jobs according to their size */
extern char *TR;		/* trailer string to be output when Q empties */
extern char *TY;		/* terminal characteristics, stty options */
extern int XC;			/* flags to clear for local mode */
extern int XS;			/* flags to set for local mode */
extern char *XT;		/* format checked for text only */
extern char *XU;		/* restrict use to users whose names are in file */

/* @@@ Fix from Dave Atkin */
extern pid_t qh_pid;            /* PID of queue handler process */
/* end fix */

/*
 * Filter_name[] is an array of filter names, indexed by the first letter of the printcap
 * tag. For example: ":if=/usr/lib/iffilter -n -a:" -> Filter_name['i'-'a'] =
 * "/usr/lib/iffilter -n -a";
 */
extern char *Filter_name[26];
#define	IF	(Filter_name['i'-'a'])
#define	OF	(Filter_name['o'-'a'])
/*
 * Prefilter_name[] is similar to Filter_name[], but for prefilter entries.
 */
extern char *Prefilter_name[26];

/*
 * this is much smaller than MAXHOSTNAMELEN, but it's needed -- plp_queue can only take
 * params <=80 chars in length, and that includes the submitting hostname.
 */
#define HOSTLEN		64
#define SHORTHOSTLEN	64

/**********************************************************************
 * Globally accessed variables and routines
 **********************************************************************/
extern char *Name;		/* program name */
extern char *Printer;		/* printer name */
extern char *Person;		/* user name */
extern char *Newjobnum;         /* A number for a command */
extern int mistake;             /* var for switching */
extern char ShortHost[HOSTLEN];	/* Host machine name */
extern char Host[HOSTLEN];	/* Host machine name (fully-qualified) */
extern char *Architecture;	/* Host machine architecture */
extern char *From;		/* client's machine name */
extern char *FQDN;		/* client's machine name (full) */
extern int Debug;		/* Debugging level */
extern int Echo_on_stdout;	/* stderr output on stdout as well */
extern char *First_name;	/* first name in printcap entry */
extern char *All_names;		/* all names in printcap entry */
extern FILE *Lfd;		/* lock file */
extern int Optind, Opterr;	/* option index, error message suppression */
extern char *Optarg;		/* option value */
extern char *Opt_flag;		/* option staring flags */
extern char *Lpdlogf, *Masterlock;
extern int Lpr_port_num;	/* port number to connect to */
extern int Maxportno;		/* = MAXPORTNO; */
extern int Minportno;		/* = MINPORTNO; */
extern char Last_errormsg[];	/* last errormessage */
extern int Request;		/* current lpd request code */
extern int Print_fd;		/* output printer file */
extern int Queue_format;	/* queue status format */
extern int Is_root;		/* is this the root user? */
extern int Is_local;		/* doing this locally or remotely? */
extern struct stat LO_statb;	/* used to stat lockfile */
extern int Errorcode;		/* exit() value on fatal termination */
extern uid_t DaemonUID;		/* daemon UID */
extern gid_t DaemonGID;		/* daemon GID */
extern int Save_copy;           /* Flag if the user wants to have a copy of his job */
                                /* in the "save-area" */
extern char buf_for_sort[];     /* tmporary buffer for sorting (in utils.c) */
                                 
extern inline char *estrcp (char *, char *, char *);
extern char *Time_str (void);
extern char *Time_str_long (void);
extern char *Time_str_hist (time_t);
extern char *Time_str_cde (time_t);
extern char *Setup_filter (int, char *);
extern int Getopt (int, char *[], char *);
extern int Getopt_lpstat (int, char *[], char *);
extern char *Decode_status (plp_status_t *);
extern int Checkperm (char *, char *, char *, int *, int *, int);

extern int   prmsline;             /* Line for acct filter to change in printer_perms */
extern short tk_interface_used;    /* Flag if the tk-lpc program is used with lpc */
extern short called_from_lprm;     /* Flag to send mail for removing jobs */
extern short called_from_one_job;  /* Flag for a call from lpc */
extern short lprm_called;          /* Falg if lprm is called form lpc */
extern short mail_to_user_for_remote_remove; /* same for remote printers */
extern short job_moved;            /* Flag if the job(s) was moved */
extern int   interval;             /* Hours of update the history file, default = 1*/
extern char  unattached_printer[]; /* Name of the unattached printer */
extern char  attached_printer[];   /* Name of the attached printer */
extern short sysv_lp;              /* Flag if the programs were called in the 
                                    * "sys v mode"
				    */

/* if in doubt -- use a typedef ! (mj) */
typedef plp_signal_t (*plp_sigfunc_t)(int) ;
extern  plp_sigfunc_t plp_signal (int, plp_sigfunc_t);


/* varargs declarations: */

#if defined(HAVE_STDARG_H) && defined(__STDC__) 
/*
 * one never knows, old Domain/OS cc 6.7 sets __STDC__ but hasn't always
 * stdarg.h, so we'd better check for this, too! (mj)
 */
#include <stdarg.h>
#define HAVE_STDARGS    /* let's hope that works everywhere (mj) */
#define VA_LOCAL_DECL   va_list ap;
#define VA_START(f)     va_start(ap, f)
#define VA_SHIFT(v,t)	;	/* no-op for ANSI */
#define VA_END          va_end(ap)
#else

#include <varargs.h>
#undef HAVE_STDARGS
#define VA_LOCAL_DECL   va_list ap;
#define VA_START(f)     va_start(ap)		/* f is ignored! */
#define VA_SHIFT(v,t)	v = va_arg(ap,t)
#define VA_END		va_end(ap)
#endif

/*******************************************************************
 *  struct plp_queue
 *    used to record entries in the spool directory for printing
 *    and status reporting.
 *  NOTE: control and data file names in spool directory have the form
 *  c f <priority> <job number> <Hostname>  <terminating 0>
 *  d f <seq>      <job number> <Hostname>  <terminating 0>
 *  1 1 1               3        HOSTLEN        1
 * eg: dfA001attila.cs.umn.edu         (attila the SUN, of course)
 * eg: cfZ001attila.cs.umn.edu         (attila the SUN, of course)
 *******************************************************************/
#define CFNAMELEN (sizeof(Host)+8)	/* control or data file name length */
/*
 * fields in the name
 */
#define STARTPR	2		/* df[] */
#define STARTID	3		/* dfX[] */
#define IDLEN	3		/* dfX[NNN] */
#define	STARTFR	6		/* dfXNNN[] */
#define MAXPARMLEN	256

struct q_file {
    char name[MAXPARMLEN];	/* Name of file to print */
    off_t size;			/* Size of file in bytes */
    int copies;			/* Number of copies to print */
};

struct plp_queue {
    char q_name[CFNAMELEN + 1];	/* name of the control file */
    time_t q_time;		/* spooled at time */
    unsigned long q_time_hires;	/* spooled at time (hi-res portion) */
    int q_priority;		/* priority */
    long q_size;		/* size of the data files */
    char q_user[32];		/* user name */
    int q_num;			/* job number */
    int q_daemon;		/* daemon for this entry */
    char *q_server;		/* server for this entry */
    char *q_options;		/* options - from J line */
    int q_held;			/* non zero if job held */
    int q_entries;		/* number of entries in q_data */
    struct q_file *q_data;	/* names of files */
};
#define	q_from	q_name[STARTFR]

extern int hostcmp (char *, char *);	/* compare hostnames */
extern void Startserver (void);		/* start a printer server */
extern inline void printstatus (int);	/* report printer status */

extern plp_signal_t cleanup (int);	/* cleanup children and files */

extern int Link_port_num (void);	/* port number for link */
extern void Link_get (void);		/* copies info from link to stdout */
extern int Link_ack (int);		/* get acknowledgement from link */
extern int Link_copy (FILE *, long, char *); /* copy file across link */
extern int Link_confirm (void);		/* get confirmation from link */
extern int Link_send (char *);		/* send info across link */
extern int Link_line (int, char *);	/* open link and send info across */
extern void Link_close (void);		/* close link */
extern int Link_open (int);		/* open link */

extern int Job_match (char *, char *);	/* match host and filename */
extern int Add_name (char *);		/* adds filename to sorted list */
extern int Find_name (char *);		/* look for job in list */
extern void sendmail (struct plp_queue *, int);
extern void Std_environ (int, char **, char **); /* set up environment */
extern void Setuplog (char *, int);	/* set up log file */
extern void servicereq (void);		/* service a request from socket */
extern void rm_recfiles (void);		/* remove received files */
extern void Startprinter (void);	/* start printer daemon */
extern int getport (char *, int);
extern int banner (int, struct plp_queue *); /* print banner */
extern int endbanner (int);             /* print end banner */
extern void Rename_cf (char *);		/* rename temp file to control file */
extern int bpread (int fd, char *buf, int size); /* bomb-proof read */
extern void splitline (char *);		/* split line into Parms */
extern void Get_Printer (int);		/* get printer name */
extern int Control_ops (void);		/* perform control operation */
extern int interpret_job (char *, char *, char *, char *, char *, char *, char *);
extern struct plp_queue *Jobentry;	/* current job entry in queue */
extern struct plp_queue *Queue;		/* Getq generates an array of jobs */
extern struct plp_queue *Queue_tmp;     /* Needed by banner without */
					/* changing the act values */
extern char *save_cfname;               /* needed by lpr.c for save copies */
extern char *save_dfname;               /* needed by lpr.c for save copies */
extern int Jobcount;			/* and returns the number of jobs  */
extern int Rec_cnt;			/* number of files in a job */

extern unsigned int Getq (void);			/* find queue entries */
extern void Remove_job (FILE *, struct plp_queue *);	/* remove job */
extern int Match_entry (struct plp_queue *);	/* check username and job number */
extern int valid_forms (char *, char *);	/* check if form is valid */
extern char *possible_options (void);		/* list of possible forms */
extern char *current_options (void);		/* current forms options */
extern long current_limit (void);
extern const char *Sigstr (int);

extern void Remote_control (char *, char *);
extern void Remote_status (void);
extern int Remote_start (void);

extern int called_by_lpr;
extern char rank_for_lpr[];
extern int Job_number_by_lpr;
extern short forms_info_printed;

extern char jobname_bb[];            /* for use with banner programs and */
extern char bnrname_bb[];            /* the BB flags used in setup_filter.c */
extern char username_bb[];
extern char seqnumber_bb[];
extern char hostname_bb[];

/********************************************************************
 * File Permissions
 ********************************************************************/

#define LFMODE      0644	/* log file permissions */
#define FILMOD      0600	/* spooling directory file permission */

/*
 * Queue Control is done by using the permissions of the lock file for the queue.  The
 * following permissions are used: 
 * Disable Printing:    owner execute or 0100 
 * Disable Queing:	group execute or 0010 
 * Queue handler active (antique) 0001
 */
#define	DISABLE_PRINT	(0100)
#define	ENABLE_PRINT	(0777 & ~ DISABLE_PRINT)
#define	DISABLE_QUEUE	(0010)
#define	ENABLE_QUEUE	(0777 & ~ DISABLE_QUEUE)
#define	FORCE_REQUE	(0001)
#define	CLEAR_REQUE	(0777 & ~ FORCE_REQUE)

/*
 * syslog(8) message priorities.  These are defined by the values of global variables.
 * The variables are initialized either with values in the <syslog.h> include file,  or
 * by a set of default values according to the NOSYSLOG option in the Makefile.
 */
extern int XLOG_ERR;		/* 4 synonym of LOG_ERROR */
extern int XLOG_CRIT;		/* 5	critical information */
extern int XLOG_WARNING;	/* 6	warning */
extern int XLOG_NOTICE;		/* 7	important information */
extern int XLOG_INFO;		/* 8	informational message */
extern int XLOG_DEBUG;		/* 9	Debug level info */

/*******************************************************************
 * Printcap Data Structures
 * The printcap file information is extracted using a single pass of
 * the printcap database.  As each entry is found,  it is looked up
 * in a table which has the entry tag,  the type of entry,
 * a default value,  and the variable used to hold the value.
 *
 * The table is sorted alphabetically by the tag values and the update
 * routines use a modified binary search.  The time taken to read the
 * printcap entry is proportional to
 * K  log( M ), where K is the length of the printcap file and M is
 * the number of printcap variables.  Note that this appears to be
 * a much simpler and easier way to implement printcap information
 * extraction.  The "initialization" of the entire set of variables
 * is extremely fast,  apparently taking about the same time  as reading
 * two variables using the original "termcap" code.
 *******************************************************************/
typedef struct pc_entry {
    char pc_name[3];		/* two character name, and the last char */
    int kind;			/* the kind of entry */

#define PC_NUM		0	/* integer */
#define PC_FLAG		1	/* integer */
#define PC_STRING	2	/* string */
#define PC_LONG		3	/* long */

    int idefault;		/* this is the default integer or flag value */
    char *sdefault;		/* this is the default integer or flag value */
    /*
     * the variable is assumed to be a pointer to char; this is cast to pointer to int if
     * neccessary.
     */
    char **var;
    /*
     * has the variable been set in a higher-level printcap entry, ie. should
     * this var be ignored in included printcap entries?
     */
    char set;
} PC_ENTRY;

/*
 * Routines to look up printer names and printcap entries char ** All_printers():  reads
 * the printcap database and extracts the names of all the printer entries. char
 * *First_printer(): gets name of first printer Get_pc_entry(): given name of a printer
 * and an array  of PC_ENTRIES, it finds the printcap entry and extracts all the
 * information for the variables listed in the array. Set_pc_entry(): same functionality,
 * does not initialize variables.
 */
#define PRNAMELEN	32	/* Maximum of 32 characters for each printer name */

#include "library/printcap.h"

/*
 * All_pc_vars[]: all printcap variables, and is used by lpr and lpd; Status_pc_vars[]: a
 * short set needed by lpq, lprm, and lpc. Server_pc_vars[]: a few needed by servers only
 */
extern PC_ENTRY All_pc_vars[ /* All_pc_len */ ];
extern int All_pc_len;
extern PC_ENTRY Status_pc_vars[ /* Status_pc_len */ ];
extern int Status_pc_len;
extern PC_ENTRY Server_pc_vars[ /* Server_pc_len */ ];
extern int Server_pc_len;

/***********************************************************************
 * File Locking Support:
 * see lockfile.c for details.
 ***********************************************************************/

extern FILE *Readlockfile (char *, pid_t *, char *, int, struct stat *);
extern FILE *Getlockfile (char *, pid_t *, char *, int, struct stat *);
extern pid_t Checklockfile (char *, pid_t *, char *, int, struct stat *);

extern void Setlockfile (char *, FILE *, pid_t, char *); /* set a lock on printer */
extern void Closelockfile (char *, FILE *); /* truncate & close lock file */

extern FILE *Lockcf (char *);	 /* lock a control file */
extern FILE *Unlockcf (char *);	 /* unlock a control file */
extern int Exlockcf (char *);	 /* create and lock a control file (exclusively) */
extern int NonExlockcf (char *); /* create and lock a control file (non-ex) */

extern int Get_seqnumber(char *);/* get a unique sequence number */

extern FILE *fopen_daemon (char *, char *); /* open a file as daemon */
extern int open_daemon (char *, int, int);  /* open a file as daemon */
extern int chmod_daemon (char *, int);	    /* change file modes */
extern int unlink_daemon (char *);	    /* delete a file */
extern int mkdir_daemon (char *, int);	    /* make a dir */

/***********************************************************************
 * Parameter List Support
 * struct parm parmlist[]
 *  is used when parsing a command received from a remote Host
 * It is also used by lpr to determine the command file, and to keep
 * track of the file/datafile name correspondence.
 ***********************************************************************/

struct parm {
    char *str;			/* string parameter */
    int num;			/* number parameter */
    char filename[CFNAMELEN + 1];	/* file name for parameter */
    long size;			/* size of file */
};
extern struct parm *Parms;	/* array of parmaters */
extern int Parmcount;		/* number of parameters */
extern unsigned int ParmSz;	/* size of the Parm array (in struct parms) */

extern void Shift_parms (int);	/* shift Parms array */
extern void allocParmsIfNeeded (void), growParmsIfNeeded (void);

/***********************************************************************
 * Request types
 * A request sent to the LPD daemon has the format:
 * \Xprinter [options],  where \X is a single character or byte value.
 * The following are the values and commands
 ***********************************************************************/

#define REQ_START	1	/* start printer */
#define REQ_RECV		2	/* transfer a job to the Host from a remote site */
#define REQ_DSHORT	3	/* print short form of queue status */
#define REQ_DLONG	4	/* print long form of queue status */
#define REQ_REMOVE	5	/* remove jobs */
#define REQ_CONTROL	6	/* do control operation */
#define REQ_DSHORT_OLD	7	/* print short form of queue status in old format */
#define REQ_DLONG_OLD	8	/* print long form of queue status in old format */
#define REQ_HISTORY	9	/* history of jobs */
/**************************************************************************
 *      Control file format
 *      First character is kind of entry, remainder of line is
 *          the argument.
 *
 *		1 -- "R font file" for troff -ignore
 *		2 -- "I font file" for troff -ignore
 *		3 -- "B font file" for troff -ignore
 *		4 -- "S font file" for troff -ignore
 *		C -- "class name" on banner page (priority)
 *		D -- "Date" job submitted
 *		H -- "Host name" of machine where lpr was done
 *		I -- "indent" amount to indent output
 *		J -- "job name" on banner page
 *		L -- "literal" user's name to print on banner
 *		M -- "mail" to user when done printing
 *		N -- "name" of file (used by lpq)
 *		P -- "Person" user's login name
 *		R -- account id  for charging
 *		U -- "unlink" name of file to remove after we print it
 *		W -- "width" page width for PR
 *		Z -- xtra options to filters
 *
 *		Lower case letters are formats
 *		f -- "file name" name of text file to print
 *		l -- "file name" text file with control chars
 *		p -- "file name" text file to print with pr(1)
 *		t -- "file name" troff(1) file to print
 *		n -- "file name" ditroff(1) file to print
 *		d -- "file name" dvi file to print
 *		g -- "file name" plot(1G) file to print
 *		v -- "file name" plain raster file to print
 *		c -- "file name" cifplot file to print
 *
 * CFparm is used to hold the upper case parameters
 */
extern char CFparm[26][MAXPARMLEN + 1];
#define CLASSNAME CFparm['C'-'A']	/* RFC: 31 char limit */
#define WHENSP    CFparm['D'-'A']	/* PLP */
#define FROMHOST  CFparm['H'-'A']	/* RFC: 31 char limit */
#define INDENT    CFparm['I'-'A']	/* RFC: number */
#define JOBNAME   CFparm['J'-'A']	/* RFC: 99 char limit */
#define BNRNAME   CFparm['L'-'A']	/* RFC: 31 char limit */
#define FILENAME  CFparm['N'-'A']	/* RFC: 131 char limit */
#define MAILNAME  CFparm['M'-'A']	/* RFC: ?? char limit */
#define LOGNAME   CFparm['P'-'A']	/* RFC: 31 char limit */
#define ACCNTNAME CFparm['R'-'A']	/* PLP: info for accounting */
#define SLINKDATA CFparm['S'-'A']	/* RFC: number " " number */
#define PRTITLE   CFparm['T'-'A']	/* RFC: 79 char limit */
#define UNLNKFILE CFparm['U'-'A']	/* RFC: flag */
#define PWIDTH    CFparm['W'-'A']	/* RFC: number */
/*
 * NOTE: for some reason, earlier versions of PLP decided to violate the
 * RFC by using T for something else, and X for the title, so
 * we can't use X for anything for a while (so we keep compatibility).
 * Argh.
 * S records data for symlinks (security); this wasn't implemented
 * in earlier versions of PLP, and S was used to suppress banner.
 * It's implemented now to avoid a security hole.
 */
#define ZOPTS     CFparm['Z'-'A']	/* PLP */

/*
 * printjob return codes
 */
#define JBUSY   -1		/* being done by other means */
#define JFAIL    0		/* failed - try later */
#define JSUCC    1		/* done */
#define JABORT   2		/* done, but problems */
#define JFULL	 3		/* remote system full - try later */
#define JNOCONF	 4		/* no confirm - assume it's been printed anyway */

/*
 * Filter exit codes
 */
#define LP_SUCCESS	0	/* printed */
#define LP_RETRY	1	/* retry printing again */
#define LP_FAILED	2	/* failed -- abort job */
#define LP_NOPAGES	100	/* out of pages */

/*
 * Mail program
 */
#ifndef MAIL
#define MAIL "/usr/lib/sendmail -oi -t"
#endif				/* MAIL */

/*
 * File Transfer Protocol Flags
 */
#define CEND	5		/* last file */
#define CNAME	4		/* control file */
#define DFILE	3		/* data file */
#define CFILE	2		/* control file */

/*
 * Local parameters to the spooling system Defaults for line printer capabilities data
 * base
 */
#define DEFLOGF		"log"
#define DEFLOCK		"lock"
#define DEFSTAT		"status"
#define DEFPRSTAT	"prstatus"
#define	DEFFORMATS	"flp"	/* default formats */
#define	DEFMX		1000
#define	DEFRETRY	3	/* maximum number of retries */
#define DEFMAXCOPIES	0
#define DEFMINLEGAL	32	/* check only first 32 chars */
#define DEFFF		"\f"
#define DEFWIDTH	80
#define DEFLENGTH	66
#define DEFPRICE	20	/* cost in dollars per thousand pages */
#define DEFPRIORITY	'X'	/* default priority level */
#define DEFPR		"/bin/pr"	/* default pr program */

#ifndef DAEMON
#define DAEMON		"daemon"/* daemon user id */
#endif

#define DEF_OPTIONS     "-"	/* to indicate that no options have been specified */

#ifdef LOG_LPR
#define SYSLOG_FACILITY	LOG_LPR
#else
#ifdef LOG_LOCAL0
#define SYSLOG_FACILITY	LOCAL0
#else
#define SYSLOG_FACILITY	(0)	/* for Ultrix -- facilities aren't supported */
#endif
#endif

/*************************************************************************
 * jmason additions from here on (v3.3.0)
 */

/* there's AF_UNIX support in there too, but what's the point of that!? */
#define SOCK_FAMILY	AF_INET

#ifndef CONFIG_FILE
#define CONFIG_FILE     "/etc/plp.conf"
#endif

/*************************************************************************
 * Edinburgh additions:
 */

extern char *FL;		/* list of valid forms */
extern int ED;			/* Expensive device - only print jobs from trusted hosts */
extern int NC;			/* Non-charged queue of expensive device */

/* Printcap configuration stuff */

struct conf_entry {
    int type;
    char *domain;
};

#define MAX_CONFS 10		/* the max. no. of entries which we look at */
extern int num_confs;
extern struct conf_entry conf[MAX_CONFS];

#ifndef PNAME_SEPCH
#define PNAME_SEPCH '/'
#endif

/* CROSS_DOMAIN is used to enable cross-domain
 * NIS & Hesiod printcap searches. (EUCS feature)
 */

#ifdef NIS
#undef CROSS_DOMAIN
#define CROSS_DOMAIN
extern int NIS_order;
#endif

#ifdef HESIOD
#undef CROSS_DOMAIN
#define CROSS_DOMAIN

#define HES_ERR_OK 0		/* Hesiod error codes */
#define HES_ERR_NOTFOUND 1
#define HES_MAXHOSTLEN 64	/* Hesiod implementation limits */
#define HES_MAXKEYLEN 256

/* if anyone has trouble getting this to compile because of the
 * lack of arg prototypes, fix it and send me a patch.
 */
extern char **hes_resolve ();	/* Hesiod printcap entries: external linkage */
extern char **hesiod_match ();	/* Hesiod routines in read_config.c */
extern int hes_error ();
extern int Hesiod_order;	/* the "order" keyword */
#endif

#ifdef CROSS_DOMAIN
extern char *out_domain;	/* name of external domain in which match occurred */
#endif

extern char *Forms_file;

/* for the printcap_*.c files: */
#define NO_CONT 0		/* don't look for continuations */
#define DO_CONT 1		/* look for continuations */

extern char *Errors_file;
extern char *Attach_file;

/*************************************************************************
 * jmason additions from here on (v3.3.1 and onwards)
 */

#define MAXUSERENVIRON  100		/* max no. of envs to keep */

#ifdef LOCAL
#define LOCAL_TYPE	1
#endif
#ifdef NIS
#define NIS_TYPE	2
#endif
#ifdef HESIOD
#define HESIOD_TYPE	3
#endif

#ifdef LOCAL
extern char *Permfile_path;
extern char *Printcap_path;
extern char *Zoptions_path;
extern char *Costcode_path;
extern int Local_order;
#endif

#ifdef HESIOD
extern int *Hesiod_cross_domain;
extern char *Hesiod_printcap_key;
extern char *Hesiod_costcode_key;
#endif

#ifdef NIS
extern char *NIS_printcap_bykey;
extern char *NIS_printcap_byname;
extern char *NIS_costcode_bykey;
extern char *NIS_costcode_byname;
#endif

extern char EUCS_banner_format;
extern char *Forms_active;

extern FILE *fopen_path (char *, char *);

extern char *ConfigFile;

extern char *Daemon_user, *Mail_command, *Mail_addressing;
extern char *Mail_from_address, *Ctrlfile_logdir, *DefaultPrintcap;
extern char *Filter_path, *Filter_LD_path, *Printer_port, *Host_domain;
extern char *Printcap_include_path, *Config_include_path;
extern char *Permfile_include_path, *AnonUsername, *SaveCopyArea;
extern char *Remote_user_auth;

extern char AllowNonPrivPorts, LogNonPrivPorts, Mail_on_errors, Restrict_symlinks;
extern char Log_no_filenames, Log_LPRMs, Log_LPQs, Log_LPCs, Log_LPRs;
extern char Check_for_nonprintable, Show_empty_queues, jobs_sorted_on_size;
extern char DomainNameUnset, Use_EUID, Use_printcap_cache, Use_perm_cache;
extern char Use_first_printer, Use_history, Use_with_cde;
extern unsigned long spooled_time, printed_time;
extern int no_bb_banner;

extern char *SafeEnviron[MAXUSERENVIRON];

/* Additions by B. Steinsbo */
#ifndef NGROUPS_MAX		/* number of simultaneous groups */
#ifdef NGROUPS
#define NGROUPS_MAX	NGROUPS
#else
#define NGROUPS_MAX	255
#endif
#endif

#ifndef GRP_NAM_MAX		/* max length of a group name */
#define GRP_NAM_MAX	15
#endif

extern uid_t UserUID;

extern char *reallocstrcpy (char *, char *, int);
#define allocandcpy_len(to,from,len)	to = reallocstrcpy(to,from,len)
#define allocandcpy(to,from)		to = reallocstrcpy(to,from,0)

extern char *Malloc_failed_msg;
/*
 * The code used to check to make sure dynamic arrays have space left and realloc to the
 * maximum size. If these are macros, it saves lots of tricky handle-passing.
 */
#define chkandrealloc_str(BUFFER,INDEX,ARYSZ,SZINCREMENT)		\
  if (BUFFER == NULL) {							\
    if (!(BUFFER = (char *) malloc(ARYSZ)))				\
    { logerr_die(XLOG_NOTICE, Malloc_failed_msg); }			\
  }									\
  while ((INDEX >= ARYSZ - 1) && (ARYSZ > 1)) {				\
    ARYSZ += SZINCREMENT;     /* try to fatten the buffer */		\
    if ((BUFFER = (char *) realloc(BUFFER, ARYSZ)) == NULL)		\
    { logerr_die(XLOG_NOTICE, Malloc_failed_msg); }			\
  }

#define chkandrealloc_list(BUFFER,INDEX,ARYSZ,SZINCREMENT)		\
  if (BUFFER == NULL) {							\
    if ((BUFFER = (char **)malloc(sizeof(char *)*ARYSZ)) == NULL)	\
    { logerr_die(XLOG_NOTICE, Malloc_failed_msg); }			\
  }									\
  while ((INDEX >= ARYSZ - 1) && (ARYSZ > 1)) {				\
    ARYSZ += SZINCREMENT;     /* try to fatten the buffer */		\
    if ((BUFFER = (char **) realloc(BUFFER,sizeof(char *)*ARYSZ)) == NULL) \
    { logerr_die(XLOG_NOTICE, Malloc_failed_msg); }			\
  }

extern void full_root_to_user (void), full_root_to_daemon (void),
	full_root_perms (void), root_to_user (void), root_to_daemon (void),
	user_to_root (void), user_to_daemon (void), daemon_to_root (void),
	daemon_to_user (void), set_user_uid (int), Setuid_debug (char *),
	full_daemon_perms (void), full_user_perms (void), root_to_uid (uid_t),
        uid_to_root (uid_t);

extern int secure_system (char *);
extern void secure_exec (char *);

extern void hires_time (unsigned long *, unsigned long *);

extern DIR *opendir_SD (void);
extern void chdir_SD (void), fix_SD (void);

extern char DbgPcap, DbgRem, DbgLocal, DbgConf, DbgPerms;

extern int Run_as_daemon;
extern int Testing_mode;
extern int ThisIsLPD;

#define malloc_or_die(x,y)	if (((x) = malloc((unsigned) y)) == 0) { \
				  logerr_die (XLOG_INFO, Malloc_failed_msg); \
				}

extern pid_t plp_waitpid (pid_t, plp_status_t *, int);	/* in startserver.c */
extern plp_signal_t sigchld_handler (int);
extern void Setup_waitpid (void);	/* for plp_waitpid() */

extern void plp_block_signals (void), plp_unblock_signals (void),
	setup_clean_fds (void), setup_close_on_exec (void),
	Parse_debug (char *);

extern void Log_cf (char *, char *, int);
extern char **copy_haddr_list (char **);
extern void free_haddr_list (char **);
extern int copy_file (char *, char *, int);

/* easier to read than strcmp(s1,s2) == 0; may be faster, too. */
#define strsame(s1, s2)		((*(s1) == *(s2)) && (strcmp((s1), (s2)) == 0))

/*************************************************************************
 * Some default values -- I think these are correct.
 */

#undef NO_SYMLINKS		/* define this to disable lpr -s */

#define YP_GET_DOMAIN_BROKEN	/* yp_get_default_domain() doesn't use malloc() for
				   return data address. */

/* these are set to default values in the NIS code. */

/* #undef YP_BIND_RETURNS */	/* if yp_bind to bad domain returns YPERR_DOMAIN */
/* #undef YP_FORK */		/* fork to implement NIS timeout because yp_bind
				   doesn't return and can't be interrupted by
				   SIGALRM */
/* #define YP_BIND_TIMEOUT	2 */	/* timeout period for yp_bind (secs) */

#ifndef ARCHITECTURE
#define ARCHITECTURE	"unknown"
#endif

#define AUTOMOUNT_STR   "/tmp_mnt/"
#undef AUTOMOUNT_SUBST

#define	True	1
#define False	0

#define ISEOL(c)                ((c) == '#' || (c) == '\n' || (c) == '\0')
#define ISSPACE(c)              ((c) == ' ' || (c) == '\t')

/* some debug macros -- our debugging verbosity is too high... */

/* miscellanous */
#define DEBUG0		if (Debug > 0) logDebug
#define DEBUG1		if (Debug > 1) logDebug
#define DEBUG2		if (Debug > 2) logDebug
#define DEBUG3		if (Debug > 3) logDebug
#define DEBUG4		if (Debug > 4) logDebug
#define DEBUG5		if (Debug > 5) logDebug
#define DEBUG6		if (Debug > 6) logDebug
#define DEBUG7		if (Debug > 7) logDebug
#define DEBUG8		if (Debug > 8) logDebug
#define DEBUG9		if (Debug > 9) logDebug

/* local printer actions */
#define DBGLOCAL0	if (DbgLocal > 0) logDebug
#define DBGLOCAL1	if (DbgLocal > 1) logDebug
#define DBGLOCAL2	if (DbgLocal > 2) logDebug
#define DBGLOCAL3	if (DbgLocal > 3) logDebug
#define DBGLOCAL4	if (DbgLocal > 4) logDebug
#define DBGLOCAL5	if (DbgLocal > 5) logDebug
#define DBGLOCAL6	if (DbgLocal > 6) logDebug
#define DBGLOCAL7	if (DbgLocal > 7) logDebug
#define DBGLOCAL8	if (DbgLocal > 8) logDebug
#define DBGLOCAL9	if (DbgLocal > 9) logDebug

/* remote printer actions */
#define DBGREM0		if (DbgRem > 0) logDebug
#define DBGREM1		if (DbgRem > 1) logDebug
#define DBGREM2		if (DbgRem > 2) logDebug
#define DBGREM3		if (DbgRem > 3) logDebug
#define DBGREM4		if (DbgRem > 4) logDebug
#define DBGREM5		if (DbgRem > 5) logDebug
#define DBGREM6		if (DbgRem > 6) logDebug
#define DBGREM7		if (DbgRem > 7) logDebug
#define DBGREM8		if (DbgRem > 8) logDebug
#define DBGREM9		if (DbgRem > 9) logDebug

/* permissions database lookups */
#define DBGPERMS0	if (DbgPerms > 0) logDebug
#define DBGPERMS1	if (DbgPerms > 1) logDebug
#define DBGPERMS2	if (DbgPerms > 2) logDebug
#define DBGPERMS3	if (DbgPerms > 3) logDebug
#define DBGPERMS4	if (DbgPerms > 4) logDebug
#define DBGPERMS5	if (DbgPerms > 5) logDebug
#define DBGPERMS6	if (DbgPerms > 6) logDebug
#define DBGPERMS7	if (DbgPerms > 7) logDebug
#define DBGPERMS8	if (DbgPerms > 8) logDebug
#define DBGPERMS9	if (DbgPerms > 9) logDebug

/* printcap database lookups */
#define DBGPCAP0	if (DbgPcap > 0) logDebug
#define DBGPCAP1	if (DbgPcap > 1) logDebug
#define DBGPCAP2	if (DbgPcap > 2) logDebug
#define DBGPCAP3	if (DbgPcap > 3) logDebug
#define DBGPCAP4	if (DbgPcap > 4) logDebug
#define DBGPCAP5	if (DbgPcap > 5) logDebug
#define DBGPCAP6	if (DbgPcap > 6) logDebug
#define DBGPCAP7	if (DbgPcap > 7) logDebug
#define DBGPCAP8	if (DbgPcap > 8) logDebug
#define DBGPCAP9	if (DbgPcap > 9) logDebug

/* plp.conf file lookups */
#define DBGCONF0	if (DbgConf > 0) logDebug
#define DBGCONF1	if (DbgConf > 1) logDebug
#define DBGCONF2	if (DbgConf > 2) logDebug
#define DBGCONF3	if (DbgConf > 3) logDebug
#define DBGCONF4	if (DbgConf > 4) logDebug
#define DBGCONF5	if (DbgConf > 5) logDebug
#define DBGCONF6	if (DbgConf > 6) logDebug
#define DBGCONF7	if (DbgConf > 7) logDebug
#define DBGCONF8	if (DbgConf > 8) logDebug
#define DBGCONF9	if (DbgConf > 9) logDebug

#endif	/* PLP_LP_H */
