#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/fcntl.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <netdb.h>
#include <termios.h>
#include <time.h>
/*
 *  PostScript program to fetch the printer's pagecount.
 *  The answer will contain a random string, which cannot
 *  be determined beforehand; so the chances of being faked
 *  by another postscript user are small.
 *  The output is produced on the stderr stream and the exit code is 0.
 *  If the exit code is != 0 than the output on stderr, signals some error.
 *
 *					Elmar Bartel, March 1994,1997
 *					bartel@informatik.tu-muenchen.de
 */
#define WAIT_FOR_ANSWER	120
#define MAX_CONNECT_TRY	 30
#define SLEEP_INTERVAL	 10

int verbose = 0;

query(int in_fd, int out_fd) {
	char istring[64], cstring[16], astring[16], *ch, *cp; 
	char OutBuffer[1024];
	char InBuffer[1024];
	int BufLeng;
	fd_set in_set;
	int fdcnt;
	static struct timeval timeout = {
	  WAIT_FOR_ANSWER, 0 
	};

	sprintf(istring, "%%%%[ Pagecount: @%d:", time(0));
	sprintf(cstring, " ]%%%%\n");

	if (isatty(in_fd))
		tcflush(in_fd, TCIFLUSH);
	sprintf(OutBuffer, "flush (%s) print statusdict begin pagecount end\
		20 string cvs print (%s) print flush\004", 
		istring, cstring);
	BufLeng= strlen(OutBuffer);
	if (write(out_fd, OutBuffer, BufLeng) != BufLeng) {
		fprintf(stderr, "cannot send query\n");
		exit(1);
	}

        FD_ZERO(&in_set);
	FD_SET(in_fd, &in_set);
	fdcnt=select(in_fd+1, (void *)&in_set, NULL, NULL, &timeout);
	if (fdcnt < 0) {
	    fprintf(stderr, "cannot select on input\n");
	    exit(2);
	}
	if (fdcnt == 0) {
	    fprintf(stderr, "no answer after %d seconds\n", timeout.tv_sec);
	    exit(3);
	}
	BufLeng= read(in_fd, InBuffer, sizeof(InBuffer)-1);
	if (BufLeng < 0) {
	    fprintf(stderr, "cannot read from input\n");
	    exit(4);
	}
	InBuffer[BufLeng]= '\0';
	ch= InBuffer;
	if (ch=strstr(InBuffer, istring)) {
	    cp= astring;
	    ch+= strlen(istring);
	    while (*ch != *cstring)
		*cp++ = *ch++;
	    *cp++= '\n';
	    *cp= '\0';
	    fputs(astring, stderr);
	    fflush(stderr);
	    exit(0);
	}
	else {
	    fputs("unexpected input: ", stderr);
	    fputs(InBuffer, stderr);
	    exit(5);
	}
}

QueryNetDevice(char *device, int MaxTry, int Pause) {
    short port= 9100;
    unsigned long addr_tmp;
    struct sockaddr_in sinme;
    struct sockaddr_in sinhim;
    struct hostent *addr;
    int  s, try;

    /* now lets try to make a connection to this device */
    memset((char *)&sinhim, 0, sizeof(sinhim));
    if (atoi(device) > 0 )  {
	sinhim.sin_family = AF_INET;
	sinhim.sin_addr.s_addr = inet_addr(device);
    }
    else {
	if ((addr=gethostbyname(device)) == NULL) {
	    fprintf(stderr, "qpc: %s %s\n", device, strerror(errno));
	    exit(9);
	}
	sinhim.sin_family = addr->h_addrtype;
	memcpy((char*)&addr_tmp, addr->h_addr, addr->h_length);
	sinhim.sin_addr.s_addr = addr_tmp;
    }
    sinhim.sin_port = htons(port);
    sinme.sin_port = 0;
    sinme.sin_family = AF_INET;

    /*
    if (bind(s, &sinhim, sizeof(sinhim)) < 0) {
	fprintf(stderr, "qpc: bind: %s\n", strerror(errno));
	exit(11);
    }
    */
    try = 0;
    while (try < MaxTry) {
	if ((s = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
	    fprintf(stderr, "qpc: socket: %s\n", strerror(errno));
	    exit(10);
	}
	if (connect(s, &sinhim, sizeof(sinhim)) < 0) {
	    if (verbose)
		fprintf(stderr, "qpc: connect failed on try %d: %s\n",
		  try, strerror(errno));
	    close(s);
	    try++;
	    sleep(2);
	}
	else
	    break;
    }
    if (try < MaxTry)
	query(s, s);
    else {
	fprintf(stderr, "qpc: couldn't get connection after %d tries\n", try);
	exit(12);
    }
}

main(int argc, char *argv[]) {
    int MaxTry=MAX_CONNECT_TRY;
    int Pause= SLEEP_INTERVAL;
    char *device= NULL;
    int ac;

    if (argc == 1)
    	query(0, 1);

    ac= 1;
    while (ac < argc) {
	if (strncmp(argv[ac], "-v", 2) == 0) {
	    verbose++;
	    ac++;
	    continue;
	}
	if (strncmp(argv[ac], "-s", 2) == 0) {
	    if (argv[ac][2] != '\0')
	    	device= argv[ac]+2;
	    else
	    	device= argv[++ac];
	    ac++;
	    continue;
	}
	if (strncmp(argv[ac], "-m", 2) == 0) {
	    if (argv[ac][2] != '\0')
	    	MaxTry= atoi(argv[ac]+2);
	    else
	    	MaxTry= atoi(argv[++ac]);
	    ac++;
	    continue;
	}
	if (strncmp(argv[ac], "-p", 2) == 0) {
	    if (argv[ac][2] != '\0')
	    	Pause= atoi(argv[ac]+2);
	    else
	    	Pause= atoi(argv[++ac]);
	    ac++;
	    continue;
	}
	fprintf(stderr, "qpc: unknown option: %s\n", argv[ac]);
	exit(6);
    }
    if (!device) {
	fprintf(stderr, "qpc: we need at least the name of the device!\n");
	exit(6);
    }
    else
    	QueryNetDevice(device, MaxTry, Pause);
}
